# coding: utf-8
from unittest import TestCase

from scenarios.scenario_models.field_requirements.field_requirements import IsIntFieldRequirement


class IsIntFieldRequirementTest(TestCase):

    @classmethod
    def setUpClass(cls):
        items = {}
        cls.requirement = IsIntFieldRequirement(items)

    def test_is_int_number_string(self):
        text = "123"
        self.assertTrue(self.requirement.check(text))

    def test_is_int_float_string(self):
        text = "1.23"
        self.assertFalse(self.requirement.check(text))

    def test_is_int_text_string(self):
        text = "test"
        self.assertFalse(self.requirement.check(text))

    def test_is_int_empty_string(self):
        text = ""
        self.assertFalse(self.requirement.check(text))



import json
import logging
import types

import requests

from smart_kit.text_preprocessing.http_text_normalizer import NormalizationError


def to_bool(value):
    if str(value).lower() == "true":
        return True

    if str(value).lower() == "false":
        return False

    if str(value).isdigit():
        value = int(value)

    return bool(value)


def as_is(value):
    return value


__typecast_map = {
    bool: to_bool,
    float: float,
    int: int,
    str: str,
}

TYPECAST_MAP = types.MappingProxyType(__typecast_map)


class TypeCastByAnnotation:
    # Чет выглядит ваще как техномагия
    def items(self):
        res = []
        for var_name in self:
            res.append((var_name, getattr(self, var_name)))
        return res

    def __setattr__(self, key: str, value):
        if key in self.__annotations__:
            _type = self.__annotations__[key]
            _type = TYPECAST_MAP.get(_type, as_is)
        else:
            _type = as_is
        value = _type(value)

        return super().__setattr__(key, value)

    def __contains__(self, item):
        return item in self.__vars__()

    def __iter__(self):
        return iter(self.__vars__())

    def __vars__(self):
        _vars = [
            k for k in self.__dir__()
            if k[:2] != '__' and type(getattr(self, k, '')).__name__ != 'method'
        ]
        return [
            var
            for var in _vars
            if not (var.startswith("_") or var == "exclude" or var in self.exclude)
        ]

    exclude = ()


class Environment(TypeCastByAnnotation):
    chat_id: str
    device_capabilities_misc: bool
    device_capabilities_screen: bool
    device_capabilities_speak: bool
    device_channel: str
    device_channel_version: str
    device_client_type: str
    device_platform_name: str
    device_platform_version: str
    device_surface_version: str
    intent: str
    message_id: int
    message_name: str
    meta_time_timestamp: int
    meta_time_timezone_id: str
    meta_time_timezone_offset_sec: int
    new_session: bool
    project_name: str
    user_channel: str
    user_id: str

    exclude = ("as_dict",)

    def __init__(self):
        from smart_kit.configs import get_app_config

        self.__message = {"original_text": ""}
        self.character_appeal = "official"
        self.character_gender = "male"
        self.character_id = "sber"
        self.character_name = "Сбер"
        self.chat_id = "1"
        self.config = get_app_config()
        self.device_capabilities_misc = True
        self.device_capabilities_screen = True
        self.device_capabilities_speak = True
        self.device_channel = "MP_SBOL_IOS"
        self.device_channel_version = "9.1"
        self.device_client_type = "RETAIL"
        self.device_platform_name = "iPhone"
        self.device_platform_type = "IOS"
        self.device_platform_version = "11.1"
        self.device_surface = "SBOL"
        self.device_surface_version = "testSurfaceVersion"
        self.intent = None
        self.message_id = 0
        self.message_name = "MESSAGE_TO_SKILL"
        self.meta_time_timestamp = 1432233446145000
        self.meta_time_timezone_id = "Europe/Moscow"
        self.meta_time_timezone_offset_sec = 10800
        self.new_session = False
        self.project_name = "test_project_name"
        self.user_channel = None
        self.user_id = "local_testing_1"

    @property
    def as_dict(self):
        return {
            "messageId": self.message_id,
            "messageName": self.message_name,
            "uuid": {
                "userChannel": self.user_channel,
                "chatId": self.chat_id,
                "userId": self.user_id,
            },
            "payload": {
                "character": {
                    "id": self.character_id,
                    "name": self.character_name,
                    "gender": self.character_gender,
                    "appeal": self.character_appeal,
                },
                "device": {
                    "platformType": self.device_platform_type,
                    "platformVersion": self.device_platform_version,
                    "surface": self.device_surface,
                    "surfaceVersion": self.device_surface_version,
                    "features": {
                        "appTypes": ["DIALOG", "WEB_APP"],
                    },
                    "capabilities": {
                        "misc": {
                            "available": self.device_capabilities_misc,
                        },
                        "screen": {
                            "available": self.device_capabilities_screen,
                        },
                        "speak": {
                            "available": self.device_capabilities_speak,
                        },
                    },
                },
                "intent": self.intent,
                "message": self.__message,
                "meta": {
                    "time": {
                        "timezone_id": self.meta_time_timezone_id,
                        "timezone_offset_sec": self.meta_time_timezone_offset_sec,
                        "timestamp": self.meta_time_timestamp,
                    },
                },
                "new_session": self.new_session,
                "personInfo": {},
                "projectName": self.project_name,
            },
        }

    @property
    def text(self):
        return self.__message["original_text"]

    @text.setter
    def text(self, value):
        from smart_kit.configs import get_app_config
        config = get_app_config()

        try:
            recognizer = config.NORMALIZER
            norm = json.loads(json.dumps(recognizer(value)))
            tpr = {'original_text': norm['original_text'],
                   'normalized_text': norm['normalized_text'],
                   'tokenized_elements_list': norm['tokenized_elements_list']}
        except (requests.exceptions.RequestException, NormalizationError) as exc:
            tpr = {"original_text": value}
            logging.getLogger(__file__).warning(
                f"\nError due connection to Text Normalizer Server at {self.config.NORMALIZER_ADDRESS}.\n"
                f"Result replaced with empty dict.\n"
                f"You see this warning because you have enabled debug mode at app_config.py.\n"
                f"Error: {exc}"
            )

        self.__message = tpr

    def __str__(self):
        return json.dumps(self.as_dict)


class Settings(TypeCastByAnnotation):
    DISPLAY_RAW: bool = False




"""
Модуль для модульного тестирования работы period_determiner
"""

from datetime import datetime, timedelta
from typing import Optional

from freezegun import freeze_time
from core.utils.period_determiner import ERROR_VALUE
from core.utils.period_determiner import UNRECOGNIZED_DATE_VALUE
from core.utils.period_determiner import extract_words_describing_period
from core.utils.period_determiner import period_determiner


@freeze_time('2021-11-23')
def test_period_determiner_1():
    # если используется форма "с месяца года по ныне"
    current_date = datetime.now()

    words_to_process = [
        'с',
        'марта',
        '2019',
        'года',
        'по',
        'ныне'
    ]
    result = period_determiner(words_to_process)
    assert result == ('01.03.2019', current_date.strftime('%d.%m.%Y'))


@freeze_time('2021-11-23')
def test_period_determiner_2():
    # если используется форма "за месяц год по сегодняшний день"
    current_date = datetime.now()

    words_to_process = [
        'марта',
        '2019',
        'года',
        'по',
        'сегодняшний',
        'день',
    ]
    result = period_determiner(words_to_process)
    assert result == ('01.03.2019', current_date.strftime('%d.%m.%Y'))


@freeze_time('2021-11-23')
def test_period_determiner_3():
    # если используется полная форма
    words_to_process = [
        'марта',
        '2019',
        'года',
        'по',
        '15',
        'июня',
        '2021',
    ]
    result = period_determiner(words_to_process)
    assert result == ('01.03.2019', '15.06.2021')


@freeze_time('2021-11-23')
def test_period_determiner_4():
    # если дата начала больше даты окончания периода
    words_to_process = [
        'марта',
        '2019',
        'года',
        'по',
        '15',
        'июня',
        '2010',
    ]
    result = period_determiner(words_to_process)
    assert result == (ERROR_VALUE, ERROR_VALUE)


@freeze_time('2021-11-23')
def test_period_determiner_5():
    # если используется форма "за несколько прошлых месяцев"
    current_date = datetime.now()

    delta_month = 4
    words_to_process = [
        str(delta_month),
        'прошлых',
        'месяца'
    ]
    result = period_determiner(words_to_process)

    month: int = 0
    year: int = 0
    if delta_month > current_date.month:
        month = 12 + current_date.month - delta_month
        year = current_date.year - 1
    else:
        month = current_date.month - delta_month
        year = current_date.year

    assert result == (
        '01.{}.{}'.format(str(month).zfill(2), year),
        current_date.strftime('%d.%m.%Y')
    )


@freeze_time('2021-11-23')
def test_period_determiner_6():
    # если используется форма "за несколько прошлых дня"
    current_date = datetime.now()

    delta_day = 4
    words_to_process = [
        str(delta_day),
        'прошлых',
        'дня'
    ]
    result = period_determiner(words_to_process)
    d = current_date - timedelta(days=delta_day)

    assert result == (
        d.strftime('%d.%m.%Y'),
        current_date.strftime('%d.%m.%Y')
    )


@freeze_time('2021-11-23')
def test_period_determiner_7():
    # если переданы год и месяц
    words_to_process = [
        '2013',
        'июнь'
    ]
    result = period_determiner(words_to_process)
    assert result == ('01.06.2013', '30.06.2013')


@freeze_time('2021-11-23')
def test_period_determiner_8():
    # если корректные даты переданы в формате dd.mm.yy и dd.mm.yyyy
    words_to_process = [
        '28.01.12',
        'до',
        '12.03.2020',
    ]
    result = period_determiner(words_to_process)
    assert result == ('28.01.2012', '12.03.2020')


@freeze_time('2021-11-23')
def test_period_determiner_9():
    # если даты передана в формате dd.mm
    current_date = datetime.now()

    words_to_process = [
        '28.01'
    ]
    result = period_determiner(words_to_process)
    assert result == ('28.01.{}'.format(current_date.year),
                      '28.01.{}'.format(current_date.year))


@freeze_time('2021-11-23')
def test_period_determiner_10():
    # если конкретный квартал ипользуется
    current_date = datetime.now()

    words_to_process = [
        '3',
        'квартал',
        '2020',
    ]
    result = period_determiner(words_to_process)
    assert result == ('01.07.2020', '30.09.2020')


@freeze_time('2021-11-23')
def test_period_determiner_11():
    # если начиная с определенного квартала
    current_date = datetime.now()

    words_to_process = [
        'с',
        '3',
        'квартала',
        '2020',
    ]
    result = period_determiner(words_to_process)
    assert result == ('01.07.2020', current_date.strftime('%d.%m.%Y'))


@freeze_time('2021-11-23')
def test_period_determiner_12():
    # если одна из дат некорректная
    words_to_process = [
        '34.01.2003',
        'до',
        '12.03.2020',
    ]
    result = period_determiner(words_to_process)
    assert result == (ERROR_VALUE, '12.03.2020')


@freeze_time('2021-11-23')
def test_period_determiner_13():
    # если передать две даты с союзом "и" между ними, то определится только последняя
    words_to_process = [
        '31.01.2003',
        'и',
        '12.03.2020',
    ]
    result = period_determiner(words_to_process)
    assert result == ('12.03.2020', '12.03.2020')


@freeze_time('2021-11-23')
def test_period_determiner_14():
    # если на обработку попадут слова не связанные с периодами времени
    words_to_process = [
        'ytgjy',
        'до',
        'sdfdasq0',
    ]
    result = period_determiner(words_to_process)
    assert result == (UNRECOGNIZED_DATE_VALUE, UNRECOGNIZED_DATE_VALUE)


@freeze_time('2021-11-23')
def test_period_determiner_15():
    # если используется только месяц
    current_date = datetime.now()

    words_to_process = [
        'март'
    ]
    result = period_determiner(words_to_process)
    assert result == ('01.03.{}'.format(current_date.year),
                      '31.03.{}'.format(current_date.year))


@freeze_time('2021-11-23')
def test_period_determiner_16():
    # если используется некорретная форма к примеру за март 28 2020 года
    words_to_process = [
        'март',
        '28',
        '2020',
        'года'
    ]
    result = period_determiner(words_to_process)
    assert result == (ERROR_VALUE, ERROR_VALUE)


@freeze_time('2021-11-23')
def test_period_determiner_17():
    # если используется некорректная форма к примеру "за март 28",
    words_to_process = [
        'март',
        '28'
    ]
    result = period_determiner(words_to_process)
    assert result == (ERROR_VALUE, ERROR_VALUE)


@freeze_time('2021-11-23')
def test_period_determiner_18():
    # если используется корректная форма к примеру "за 28 марта",
    # то определится как 28 марта текущего года
    current_date = datetime.now()

    words_to_process = [
        '28',
        'марта',
    ]
    result = period_determiner(words_to_process)
    assert result == ('28.03.{}'.format(current_date.year),
                      '28.03.{}'.format(current_date.year))


@freeze_time('2021-11-23')
def test_period_determiner_19():
    # если используется корректная форма к примеру "за 28 марта",
    # то определится как 28 марта текущего года
    words_to_process = [
        '28',
        'марта',
        '2019'
    ]
    result = period_determiner(words_to_process)
    assert result == ('28.03.2019', '28.03.2019')


@freeze_time('2021-11-23')
def test_period_determiner_20():
    # если используется корректная форма к примеру "с 28 марта 2019",
    # то период определится как  с 28 марта 2019 года по сегодня
    current_date = datetime.now()

    words_to_process = [
        'с',
        '28',
        'марта',
        '2019',
        'года'
    ]
    result = period_determiner(words_to_process)
    assert result == ('28.03.2019', current_date.strftime('%d.%m.%Y'))


@freeze_time('2021-11-23')
def test_period_determiner_21():
    # если используется корректная форма к примеру "за n года",
    # то период определится как с даты ранее на 365 * n дней текущего дня
    current_date = datetime.now()

    count_of_years: int = 3
    words_to_process = [
        'за',
        str(count_of_years),
        'года'
    ]
    result = period_determiner(words_to_process)
    d1: datetime = current_date - timedelta(365 * count_of_years)
    assert result == (
        d1.strftime('%d.%m.%Y'),
        current_date.strftime('%d.%m.%Y')
    )


@freeze_time('2021-11-23')
def test_period_determiner_22():
    # если используется корректная форма к примеру "за n месяца",
    # то период определится как с даты ранее на 30 * n дней текущего дня
    current_date = datetime.now()

    count_of_months: int = 2
    words_to_process = [
        'за',
        str(count_of_months),
        'месяца'
    ]
    result = period_determiner(words_to_process)
    d1: datetime = current_date - timedelta(30 * count_of_months)
    assert result == (
        d1.strftime('%d.%m.%Y'),
        current_date.strftime('%d.%m.%Y')
    )


@freeze_time('2021-11-23')
def test_period_determiner_23():
    # если используется форма "с 2 по 17 июня"
    current_date = datetime.now()

    count_of_months: int = 2
    words_to_process = [
        'с',
        '2',
        'по',
        '17',
        'января'
    ]
    result = period_determiner(words_to_process)
    assert result == ('02.01.{}'.format(current_date.year),
                      '17.01.{}'.format(current_date.year))


@freeze_time('2021-11-23')
def test_period_determiner_24():
    # тест контроля максимального количества дней в периоде
    count_of_years: int = 2
    words_to_process = [
        'за',
        str(count_of_years),
        'года'
    ]
    result = period_determiner(words_to_process, max_days_in_period=365)
    assert result == (ERROR_VALUE, ERROR_VALUE)


@freeze_time('2021-11-23')
def test_period_determiner_25():
    # если используется корректная форма к примеру "за n недель",
    # то период определится как с даты ранее на 7 * n дней текущего дня
    current_date = datetime.now()

    count_of_weeks: int = 3
    words_to_process = [
        'за',
        str(count_of_weeks),
        'недели'
    ]
    result = period_determiner(words_to_process)
    d1: datetime = current_date - timedelta(7 * count_of_weeks)
    assert result == (
        d1.strftime('%d.%m.%Y'),
        current_date.strftime('%d.%m.%Y')
    )


@freeze_time('2021-11-23')
def test_period_determiner_26():
    # если используется корректная форма к примеру "за последние n дней",
    # то период определится как с даты ранее на n дней текущего дня
    current_date = datetime.now()

    count_of_days: int = 30
    words_to_process = [
        'за',
        'последние',
        str(count_of_days),
        'дней'
    ]
    result = period_determiner(words_to_process)
    d1: datetime = current_date - timedelta(count_of_days)
    assert result == (
        d1.strftime('%d.%m.%Y'),
        current_date.strftime('%d.%m.%Y')
    )


@freeze_time('2021-11-23')
def test_period_determiner_27():
    # когда используется только номер года и сам год
    words_to_process = [
        'за',
        '2020',
        'год'
    ]
    result = period_determiner(words_to_process)
    assert result == ('01.01.2020', '31.12.2020')


@freeze_time('2021-11-23')
def test_period_determiner_28():
    # если используется корректная форма "за неделю"
    # без указания количества недель
    current_date = datetime.now()

    words_to_process = [
        'неделю'
    ]
    result = period_determiner(words_to_process)
    d1: datetime = current_date - timedelta(7)
    assert result == (
        d1.strftime('%d.%m.%Y'),
        current_date.strftime('%d.%m.%Y')
    )


@freeze_time('2021-11-23')
def test_period_determiner_29():
    # если используется форма "за квартал",
    # без указания номера квартала, то имеем ввиду текущий квартал
    current_date = datetime.now()

    words_to_process = [
        'квартал'
    ]

    qvartal_nomer = 1
    if current_date.month in [1, 2, 3]:
        qvartal_nomer = 1
    elif current_date.month in [4, 5, 6]:
        qvartal_nomer = 2
    elif current_date.month in [7, 8, 9]:
        qvartal_nomer = 3
    else:
        qvartal_nomer = 4

    d1 = datetime(
        year=current_date.year,
        month=3 * qvartal_nomer - 2,
        day=1
    )

    d2: Optional[datetime] = None
    if qvartal_nomer == 4:
        d2 = datetime(
            year=current_date.year + 1,
            month=1,
            day=1
        ) - timedelta(days=1)
    else:
        d2 = datetime(
            year=current_date.year,
            month=3 * qvartal_nomer + 1,
            day=1
        ) - timedelta(days=1)

    result = period_determiner(words_to_process, future_days_allowed=True)
    assert result == (
        d1.strftime('%d.%m.%Y'),
        d2.strftime('%d.%m.%Y')
    )


@freeze_time('2021-11-23')
def test_period_determiner_30():
    # если даты передана в формате d.m
    current_date = datetime.now()

    words_to_process = [
        '8.1'
    ]
    result = period_determiner(words_to_process)
    assert result == ('08.01.{}'.format(current_date.year),
                      '08.01.{}'.format(current_date.year))


@freeze_time('2021-11-23')
def test_period_determiner_31():
    # если используется корректная форма к примеру "с 2019 года",
    # то период определится как с начала 2019 года по сегодня
    current_date = datetime.now()

    words_to_process = [
        'с',
        '2019',
        'года'
    ]
    result = period_determiner(words_to_process)
    assert result == ('01.01.2019', current_date.strftime('%d.%m.%Y'))


def test_extract_words_describing_period_1():
    words_from_intent = [
        "заказать",
        "выписку",
        "за",
        "3",
        "квартал",
        "2020",
        "года"
    ]
    result = extract_words_describing_period(words_from_intent)
    assert result == ['3', 'квартал', '2020', 'года']


def test_extract_words_describing_period_2():
    words_from_intent = [
        "заказать",
        "выписку",
        "с",
        "марта",
        "2019",
        "года",
        "по",
        "ныне"
    ]
    result = extract_words_describing_period(words_from_intent)
    assert result == ['с', 'марта', '2019', 'года', 'по', 'ныне']


def test_extract_words_describing_period_3():
    words_from_intent = [
        "заказать",
        "за",
        "4",
        "прошлых",
        "месяца",
        "выписку"
    ]
    result = extract_words_describing_period(words_from_intent)
    assert result == ['4', 'прошлых', 'месяца']



from twisted.internet import reactor
from twisted.web import server
from core.logging.logger_utils import log
import core.logging.logger_constants as log_const


class TwistedServer:
    def __init__(self, port, interface, handler, debug=False):
        log("TwistedServer.__init__ started.", params={log_const.KEY_NAME: log_const.TWISTED_SERVER},
                      level="WARNING")
        site = server.Site(handler(debug=debug))
        reactor.listenTCP(port, site, interface=interface or "")
        reactor.startRunning(False)
        log("TwistedServer.__init__ finished.", params={log_const.KEY_NAME: log_const.TWISTED_SERVER},
                      level="WARNING")

    def iterate(self):
        reactor.iterate()



ANSWER_TO_USER = "ANSWER_TO_USER"
ERROR = "ERROR"
NOTHING_FOUND = "NOTHING_FOUND"
MESSAGE_TO_SKILL = "MESSAGE_TO_SKILL"
LOCAL_TIMEOUT = "LOCAL_TIMEOUT"
RUN_APP = "RUN_APP"
CLOSE_APP = "CLOSE_APP"
SERVER_ACTION = "SERVER_ACTION"
GET_PROFILE_DATA = "GET_PROFILE_DATA"
TAKE_PROFILE_DATA = "TAKE_PROFILE_DATA"



# coding: utf-8
import unittest

from smart_kit.management import base


class MyCommand:
    def __init__(self, command_):
        self.executer = command_

    def execute(self, *args, **kwargs):
        return self.executer(*args, **kwargs)


class ManagementTest1(unittest.TestCase):
    def setUp(self):
        self.test_command = {"any command": ""}

    def test_app_command(self):
        with self.assertRaises(NotImplementedError):
            obj1 = base.AppCommand()
            obj1.execute((1, 2,), {"a": 1, "b": 2})

    def test_help_command_init(self):
        obj1 = base.HelpCommand(self.test_command)
        self.assertTrue(obj1._commands == self.test_command)

    def test_help_command_execute(self):
        obj1 = base.HelpCommand(self.test_command)
        # сценарий первый - not arg
        obj1.execute(tuple())
        # сценарий второй - len(arg) >= 2
        with self.assertRaises(ValueError):
            obj1.execute((1, 2), {})
        with self.assertRaises(ValueError):
            obj1.execute((1,), {})
        # сценарий третий - без включения
        obj1.execute(("bad command",))
        # сценарий четвёртый - с включением
        obj1.execute(("any command",))

    def test_manager_init(self):
        obj1 = base.Manager()
        self.assertTrue(obj1.commands == {})

    def test_manager_register_command(self):
        obj1 = base.Manager()
        obj1.register_command("str to int", MyCommand, int)
        self.assertTrue("str to int" in obj1.commands)

    def test_manager_execute_command(self):
        obj1 = base.Manager()
        obj1.register_command("str to int", MyCommand, int)
        obj1.execute_command("str to int", "221")
        with self.assertRaises(KeyError):
            obj1.execute_command("str to float", 2.88)

    def test_manager_execute_from_command_line(self):
        obj1 = base.Manager()
        obj1.register_command("str to int", MyCommand, int)
        # Случай 1: len(args) <= 1
        obj1.execute_from_command_line(('star point',))
        # Случай 2: имени нет в словаре команд
        obj1.execute_from_command_line(('star point', "str to float"))
        # Случай 3: имя есть в словаре команд
        self.assertTrue(obj1.execute_from_command_line(('star point', 'str to int', '222')) == 222)



from core.model.registered import Registered
from smart_kit.message.smartapp_to_message import SmartAppToMessage

to_messages = Registered()


def get_to_message(command_name):
    default = SmartAppToMessage
    return to_messages.get(command_name, default)



from unittest import TestCase

from core.unified_template.currency2text import Money2Text


class TestMoney2Text(TestCase):
    def setUp(self) -> None:
        self.money2text = Money2Text()

    def test_shows_currency_names(self):
        self.assertEqual('2 рубля', self.money2text(2, "RUR"))
        self.assertEqual('11 долларов США', self.money2text(11.0, 'USD'))
        self.assertEqual('12 евро', self.money2text(12, "EUR"))
        self.assertEqual('2 юаня', self.money2text(2, "CNY"))
        self.assertEqual('2 фунта стерлингов', self.money2text(2, "GBP"))

    def test_shows_diminutives(self):
        self.assertEqual('11 долларов США 11 центов', self.money2text(11.11111111, 'USD'))
        self.assertEqual('14 рублей 5 копеек', self.money2text(14.05, "RUR"))
        self.assertEqual('2 юаня 68 фынь', self.money2text(2.68, "CNY"))
        self.assertEqual('3 фунта стерлингов 40 пенсов', self.money2text(3.4, "GBP"))

    def test_can_inflect_special_cases(self):
        self.assertEqual('1 чешская крона', self.money2text(1, "CZK"))
        self.assertEqual('2 чешские кроны', self.money2text(2, "CZK"))
        self.assertEqual('5 чешских крон 50 геллеров', self.money2text(5.5, "CZK"))

    def test_shows_currency_code_if_currency_unknown(self):
        self.assertEqual('11 XXX', self.money2text(11, 'XXX'))



# coding: utf-8
from typing import Dict, Any


class Command:
    def __init__(self, name=None, params=None, action_id=None, request_type=None, request_data=None, loader=None,
                 need_payload_wrap=True, need_message_name=True):
        """
        Initialize Command instance with params

        :param name: str, command name
        :param params:
        :param action_id:
        :param request_type:
        :param request_data:
        :param loader: loader name for data before send. Possible loader values: json.dumps / protobuf
        """

        self.name: str = name
        self.payload: Dict[str, Any] = params or {}
        self.action_id: str = action_id
        self.request_type: str = request_type
        self.request_data: Dict[str, Any] = request_data or {}
        self.loader: str = loader or "json.dumps"
        self.need_payload_wrap: bool = need_payload_wrap
        self.need_message_name: bool = need_message_name

    @property
    def raw(self):
        message: Dict[str, Any] = {"payload": self.payload} if self.need_payload_wrap else self.payload
        if self.need_message_name:
            message["messageName"] = self.name
        return message



from typing import Optional

def pan2service(pan: str) -> Optional[str]:

    if len(pan) < 4:
        return None

    first_digit = int(pan[0:2])

    if first_digit == 50 or first_digit in range(56, 69):
        return "maestro"
    if first_digit in range(51, 55):
        return "mastercard"
    if first_digit == 37:
        return "amex"

    first_digit = int(pan[0])
    if first_digit == 4:
        return "visa"
    if first_digit == 2:
        return "mir"

    return None



# coding: utf-8

import logging
import os
import time
import uuid

from confluent_kafka import Consumer, TIMESTAMP_NOT_AVAILABLE
from confluent_kafka.cimpl import KafkaError, KafkaException, OFFSET_END, Message as KafkaMessage

import core.logging.logger_constants as log_const
from core.logging.logger_utils import log
from core.monitoring.monitoring import monitoring
from core.mq.kafka.base_kafka_consumer import BaseKafkaConsumer


class KafkaConsumer(BaseKafkaConsumer):
    def __init__(self, config):
        self._config = config["consumer"]
        self.assign_offset_end = self._config.get("assign_offset_end", False)
        conf = self._config["conf"]
        conf.setdefault("group.id", str(uuid.uuid1()))
        self.autocommit_enabled = conf.get("enable.auto.commit", True)
        internal_log_path = self._config.get("internal_log_path")
        conf["error_cb"] = self._error_callback
        if internal_log_path:
            debug_logger = logging.getLogger("debug_consumer")
            timestamp = time.strftime("_%d%m%Y_")
            debug_logger.addHandler(logging.FileHandler(
                "{}/kafka_consumer_debug{}{}.log".format(internal_log_path, timestamp, os.getpid())))
            conf["logger"] = debug_logger
        self._consumer = Consumer(**conf)

    @staticmethod
    def on_assign_offset_end(consumer, partitions):
        for p in partitions:
            p.offset = OFFSET_END
        KafkaConsumer.on_assign_log(consumer, partitions)
        consumer.assign(partitions)

    @staticmethod
    def on_coop_assign_offset_end(consumer, partitions):
        for p in partitions:
            p.offset = OFFSET_END
        KafkaConsumer.on_assign_log(consumer, partitions)
        consumer.incremental_assign(partitions)

    @staticmethod
    def on_assign_log(consumer, partitions):
        log_level = "WARNING"
        for p in partitions:
            if p.error:
                log_level = "ERROR"
        params = {
            "partitions": str(list([str(partition) for partition in partitions or []])),
            log_const.KEY_NAME: log_const.KAFKA_ON_ASSIGN_VALUE,
            "log_level": log_level
        }
        log("KafkaConsumer.subscribe<on_assign>: assign %(partitions)s %(log_level)s", params=params, level=log_level)

    def subscribe(self, topics=None):
        topics = topics or list(self._config["topics"].values())

        params = {
            "topics": topics
        }
        log("Topics to subscribe: %(topics)s", params=params)

        self._consumer.subscribe(topics,
                                 on_assign=self.get_on_assign_callback() if self.assign_offset_end else KafkaConsumer.on_assign_log)

    def get_on_assign_callback(self):
        if "cooperative" in self._config["conf"].get("partition.assignment.strategy", ""):
            callback = KafkaConsumer.on_coop_assign_offset_end
        else:
            callback = KafkaConsumer.on_assign_offset_end
        return callback

    def unsubscribe(self):
        self._consumer.unsubscribe()

    def poll(self):
        msg = self._consumer.poll(self._config["poll_timeout"])
        if msg is not None:
            return self._process_message(msg)

    def consume(self, num_messages: int = 1):
        messages = self._consumer.consume(num_messages=num_messages, timeout=self._config["poll_timeout"])
        for msg in messages:
            yield self._process_message(msg)

    def commit_offset(self, msg):
        if msg is not None:
            if self.autocommit_enabled:
                self._consumer.store_offsets(msg)
            else:
                self._consumer.commit(msg, **{"async": True})

    def get_msg_create_time(self, mq_message):
        timestamp_type, timestamp = mq_message.timestamp()
        return timestamp if timestamp_type is not TIMESTAMP_NOT_AVAILABLE else None

    def _error_callback(self, err):
        params = {
            "error": str(err),
            log_const.KEY_NAME: log_const.EXCEPTION_VALUE
        }
        log("KafkaConsumer: Error: %(error)s", params=params, level="WARNING")
        monitoring.got_counter("kafka_consumer_exception")

    # noinspection PyMethodMayBeStatic
    def _process_message(self, msg: KafkaMessage):
        err = msg.error()
        if err:
            if err.code() == KafkaError._PARTITION_EOF:
                return None
            else:
                monitoring.got_counter("kafka_consumer_exception")
                params = {
                    "code": err.code(),
                    "pid": os.getpid(),
                    "topic": msg.topic(),
                    "partition": msg.partition(),
                    log_const.KEY_NAME: log_const.EXCEPTION_VALUE
                }
                log(
                    "KafkaConsumer Error %(code)s at pid %(pid)s: topic=%(topic)s partition=[%(partition)s]\n",
                    params=params, level="WARNING")
                raise KafkaException(err)

        if msg.value():
            if msg.headers() is None:
                msg.set_headers([])
            return msg

    def close(self):
        self._consumer.close()
        log(f"consumer to topics {self._config['topics']} closed.")



from core.basic_models.parametrizers.parametrizer import BasicParametrizer


class Parametrizer(BasicParametrizer):

    def __init__(self, user, items):
        super(Parametrizer, self).__init__(user, items)

    def _get_scenario(self):
        scenario_id = self._user.last_scenarios.last_scenario_name
        return self._user.descriptions["scenarios"].get(scenario_id) if scenario_id else None

    def _get_main_form(self, forms):
        scenario = self._get_scenario()
        if scenario:
            return forms[scenario.form_type]
        return None

    def _get_user_data(self, text_preprocessing_result=None):
        tpr_data = text_preprocessing_result.raw if text_preprocessing_result else {}
        forms = self._user.forms.collect_form_fields()
        main_form = self._get_main_form(forms)
        data = {
            "counters": self._user.counters.raw,
            "forms": forms,
            "gender_sensitive_text": self._user.gender_selector.get_text_by_key,
            "local_vars": self._user.local_vars.values,
            "main_form": main_form,
            "message": self._user.message,
            "payload": self._user.message.payload,
            "scenario_id": self._user.last_scenarios.last_scenario_name,
            "text_preprocessing_result": tpr_data,
            "uuid": self._user.message.uuid,
            "variables": self._user.variables.values,
            "settings": self._user.settings
        }
        return data



# coding: utf-8
from core.descriptions.smart_updatable_descriptions_items import SmartUpdatableDescriptionsItems
from scenarios.behaviors.behavior_description import BehaviorDescription


class BehaviorDescriptions(SmartUpdatableDescriptionsItems):
    def __init__(self, items):
        super(BehaviorDescriptions, self).__init__(BehaviorDescription, items)



__all__ = [
    "HistoryConstants"
]


class HistoryEventTypes:
    FIELD_EVENT = "field_event"
    END_SCENARIO = "end_scenario"

class HistoryContentFields:
    FIELD = "field"


class HistoryEventResults:
    FILLED = "filled"
    ASK_QUESTION = "ask_question"
    SUCCESS = "success"


class HistoryConstants:
    types = HistoryEventTypes
    event_results = HistoryEventResults
    content_fields = HistoryContentFields



# coding: utf-
from typing import List, Dict, Any

import scenarios.logging.logger_constants as log_const
from core.basic_models.actions.command import Command
from core.logging.logger_utils import log
from core.text_preprocessing.preprocessing_result import TextPreprocessingResult
from scenarios.user.user_model import User

from smart_kit.handlers.handler_base import HandlerBase
from smart_kit.models.dialogue_manager import DialogueManager


class HandlerText(HandlerBase):
    def __init__(self, app_name: str, dialogue_manager: DialogueManager):
        super().__init__(app_name)
        log(
            f"{self.__class__.__name__}.__init__ started.", params={log_const.KEY_NAME: log_const.STARTUP_VALUE}
        )
        self.dialogue_manager = dialogue_manager
        log(
            f"{self.__class__.__name__}.__init__ finished.", params={log_const.KEY_NAME: log_const.STARTUP_VALUE}
        )

    def run(self, payload: Dict[str, Any], user: User) -> List[Command]:
        commands = super().run(payload, user)

        text_preprocessing_result = TextPreprocessingResult.from_payload(payload)

        params = {
            log_const.KEY_NAME: log_const.NORMALIZED_TEXT_VALUE,
            "normalized_text": str(text_preprocessing_result.raw),
        }
        log("text preprocessing result: '%(normalized_text)s'", user, params)

        commands.extend(self._handle_base(text_preprocessing_result, user))
        return commands

    def _handle_base(self, text_preprocessing_result: TextPreprocessingResult, user: User) -> List[Command]:
        answer, is_answer_found = self.dialogue_manager.run(text_preprocessing_result, user)
        return answer or []



from math import modf
import pymorphy2
from pymorphy2.shapes import restore_capitalization

CURRENCY_CODE_2_NAME_MAP = {
    "USD": "доллар США",
    "EUR": "евро",
    "RUR": "рубль",
    "RUB": "рубль",
    "CNY": "юань",
    "GBP": "фунт стерлингов",
    "JPY": "иена",
    "CHF": "швейцарский франк",
    "CAD": "канадский доллар",
    "SGD": "сингапурский доллар",
    "HKD": "гонконгский доллар",
    "DKK": "датская крона",
    "NOK": "норвежская крона",
    "SEK": "шведская крона",
    "AUD": "австралийский доллар",
    "UAH": "украинская гривна",
    "AZN": "азербайджанский манат",
    "DZD": "алжирский динар",
    "ARS": "аргентинское песо",
    "AMD": "армянский драм",
    "AWG": "арубанский флорин",
    "AFN": "афгани",
    "PAB": "бальбоа",
    "BBD": "барбадосский доллар",
    "THB": "таиландский бат",
    "BHD": "бахрейнский динар",
    "BZD": "белизский доллар",
    "BYN": "белорусский рубль",
    "BMD": "бермудский доллар",
    "BGN": "болгарский лев",
    "VES": "венесуэльский боливар соберано",
    "BOB": "боливиано",
    "BRL": "бразильский реал",
    "BND": "брунейский доллар",
    "BIF": "бурундийский франк",
    "VUV": "вату",
    "KRW": "вона",
    "XCD": "восточно-карибский доллар",
    "GYD": "гайанский доллар",
    "GHS": "ганский седи",
    "GNF": "гвинейский франк",
    "GIP": "гибралтарский фунт",
    "PYG": "гуарани",
    "HTG": "гурд",
    "GMD": "даласи",
    "MKD": "македонский денар",
    "JEP": "джерсийский фунт",
    "AED": "дирхам ОАЭ",
    "STN": "добра",
    "ZWL": "доллар зимбабве",
    "NAD": "доллар намибии",
    "KYD": "доллар островов кайман",
    "SBD": "доллар Cоломоновых островов",
    "TTD": "доллар Тринидада и Тобаго",
    "FJD": "доллар Фиджи",
    "DOP": "доминиканское песо",
    "VND": "донг",
    "EGP": "египетский фунт",
    "COU": "единица реальной стоимости",
    "ZMW": "замбийская квача",
    "PLN": "польский злотый",
    "NIO": "никарагуанская золотая кордоба",
    "INR": "индийская рупия",
    "JOD": "иорданский динар",
    "IQD": "иракский динар",
    "IRR": "иранский риал",
    "ISK": "исландская крона",
    "YER": "йеменский риал",
    "QAR": "катарский риал",
    "AOA": "ангольская кванза",
    "KES": "кенийский шиллинг",
    "GTQ": "гватемальский кетсаль",
    "PGK": "кина",
    "COP": "колумбийское песо",
    "KMF": "коморский франк",
    "BAM": "конвертируемая марка",
    "CUC": "конвертируемое песо",
    "CDF": "конголезский франк",
    "CRC": "коста-риканский колон",
    "CUP": "кубинское песо",
    "KWD": "кувейтский динар",
    "HRK": "хорватская куна",
    "MMK": "кьят",
    "LAK": "лаосский кип",
    "GEL": "грузинский лари",
    "ALL": "албанский лек",
    "HNL": "гондурасская лемпира",
    "SLL": "леоне",
    "LRD": "либерийский доллар",
    "LBP": "ливанский фунт",
    "LYD": "ливийский динар",
    "SZL": "лилангени",
    "LSL": "лоти",
    "MUR": "маврикийская рупия",
    "MWK": "малавийская квача",
    "MGA": "малагасийский ариари",
    "MYR": "малайзийский ринггит",
    "MAD": "марокканский дирхам",
    "MXN": "мексиканское песо",
    "MZN": "мозамбикский метикал",
    "MDL": "молдавский лей",
    "NGN": "найра",
    "ERN": "накфа",
    "BTN": "нгултрум",
    "NPR": "непальская рупия",
    "ANG": "нидерландский антильский гульден",
    "NZD": "новозеландский доллар",
    "ILS": "новый израильский шекель",
    "TWD": "новый тайваньский доллар",
    "TMT": "новый туркменский манат",
    "OMR": "оманский риал",
    "TOP": "паанга",
    "PKR": "пакистанская рупия",
    "MOP": "патака",
    "BWP": "пула",
    "KHR": "риель",
    "RON": "румынский лей",
    "IDR": "индонезийская рупия",
    "MVR": "мальдивская руфия",
    "ZAR": "южноафриканский рэнд",
    "SVC": "сальвадорский колон",
    "SAR": "саудовский риял",
    "KPW": "северокорейская вона",
    "SCR": "сейшельская рупия",
    "RSD": "сербский динар",
    "SYP": "сирийский фунт",
    "PEN": "перуанский соль",
    "KGS": "киргизский сом",
    "SOS": "сомалийский шиллинг",
    "TJS": "сомони",
    "SDG": "суданский фунт",
    "SRD": "суринамский доллар",
    "BDT": "бангладешская така",
    "WST": "тала",
    "TZS": "танзанийский шиллинг",
    "KZT": "казахстанский тенге",
    "MNT": "монгольский тугрик",
    "TND": "тунисский динар",
    "TRY": "турецкая лира",
    "UGX": "угандийский шиллинг",
    "MRU": "мавританская угия",
    "UZS": "узбекский сум",
    "UYU": "уругвайское песо",
    "PHP": "филиппинское песо",
    "HUF": "венгерский форинт",
    "DJF": "франк джибути",
    "XAF": "франк КФА ВЕАС",
    "XOF": "франк КФА ВСЕАО",
    "XPF": "франк КФП",
    "RWF": "франк Руанды",
    "IMP": "фунт Острова Мэн",
    "SHP": "фунт Святой Елены",
    "FKP": "фунт Фолклендских островов",
    "CZK": "чешская крона",
    "CLP": "чилийское песо",
    "LKR": "шри-ланкийская рупия",
    "CVE": "эскудо Кабо-Верде",
    "ETB": "эфиопский быр",
    "SSP": "южносуданский фунт",
    "JMD": "ямайский доллар",
    "XDR": "специальные права заимствования",
    "DEM": "немецкая марка",
    "ITL": "итальянская лира",
    "LVL": "латвийский лат",
    "ATS": "австрийский шиллинг",
    "ESP": "испанская песета",
    "FRF": "французский франк",
    "BEF": "бельгийский франк",
    "GRD": "греческая драхма",
    "IEP": "ирландский фунт",
    "LTL": "литовский лит",
    "CYP": "кипрский фунт",
    "LUF": "люксембургский франк",
    "MTL": "мальтийская лира",
    "NLG": "голландский гульден",
    "PTE": "португальское эскудо",
    "FIM": "финская марка",
    "ADP": "андоррская песета",
    "MTP": "мальтийский фунт",
    "SKK": "словацкая крона",
    "SIT": "словенский толар",
    "YUM": "югославский динар",
    "EEK": "эстонская крона",
    "GWE": "гвинейский эскудо",
    "GWP": "песо Гвинеи-Бисау",
    "MLF": "малийский франк",
    "TPE": "тиморское эскудо",
    "ECS": "сукре",
    "ARA": "аргентинский аустраль",
    "BOP": "боливийское песо",
    "BAD": "динар Боснии и Герцеговины",
    "BRC": "крузадо",
    "BRN": "новый крузадо",
    "BRE": "крузейро",
    "BRR": "крузейро реал",
    "GNS": "сили",
    "ZRZ": "заир",
    "ZRN": "новый заир",
    "ILP": "израильский фунт",
    "YDD": "йеменский динар",
    "MGF": "малагасийский франк",
    "MZE": "мозамбикский эскудо",
    "SDD": "суданский динар",
    "SRG": "суринамский гульден",
    "TJR": "таджикский рубль",
    "UAK": "карбованец",
    "HRD": "хорватский динар",
    "SUR": "советский рубль"
}

CURRENCY_CODE_2_DEMUNITIVE_NAME_MAP = {
    "USD": "цент",
    "EUR": "евроцент",
    "RUR": "копейка",
    "SEK": "эре",
    "DKK": "эре",
    "NOK": "эре",
    "GBP": "пенс",
    "CHF": "сантим",
    "CAD": "цент",
    "AUD": "цент",
    "SGD": "цент",
    "JPY": "сен",
    "CNY": "фынь",
    "UAH": "копейка",
    "BYN": "копейка",
    "CZK": "геллер",
    "EGP": "пиастр",
    "TRY": "куруш",
    "GEL": "тетри",
    "AMD": "лум",
    "AED": "филс",
    "AFN": "пул",
    "ALL": "киндарка",
    "ANG": "цент",
    "AOA": "сентимо",
    "ARS": "сентаво",
    "AWG": "цент",
    "AZN": "гяпик",
    "BAM": "фенинг",
    "BBD": "цент",
    "BDT": "пойша",
    "BGN": "стотинка",
    "BIF": "сантим",
    "BMD": "цент",
    "BND": "сен",
    "BOB": "сентаво",
    "BRL": "сентаво",
    "BTN": "четрум",
    "BWP": "тхебе",
    "BZD": "цент",
    "CDF": "сантим",
    "CLP": "сентаво",
    "COP": "сентаво",
    "CRC": "сентимо",
    "CUC": "сентаво",
    "CUP": "сентаво",
    "CVE": "сентаво",
    "DJF": "сантим",
    "DOP": "сентаво",
    "DZD": "сантим",
    "ERN": "цент",
    "ETB": "цент",
    "FJD": "цент",
    "FKP": "пенс",
    "GHS": "песева",
    "GIP": "пенс",
    "GMD": "бутут",
    "GNF": "сантим",
    "GTQ": "сентаво",
    "GYD": "цент",
    "HKD": "цент",
    "HNL": "сентаво",
    "HRK": "липа",
    "HTG": "сантим",
    "HUF": "филлер",
    "IDR": "сен",
    "ILS": "агор",
    "IMP": "пенс",
    "INR": "пайса",
    "IRR": "динар",
    "JEP": "пенс",
    "JMD": "цент",
    "JOD": "пиастр",
    "KES": "цент",
    "KGS": "тыйын",
    "KHR": "сен",
    "KMF": "сантим",
    "KPW": "чон",
    "KRW": "чон",
    "KYD": "цент",
    "KZT": "тиын",
    "LAK": "ат",
    "LBP": "пиастр",
    "LKR": "цент",
    "LRD": "цент",
    "LSL": "сенте",
    "MAD": "сантим",
    "MDL": "бань",
    "MKD": "дени",
    "MMK": "пья",
    "MNT": "мунгу",
    "MOP": "аво",
    "MUR": "цент",
    "MVR": "лари",
    "MWK": "тамбала",
    "MXN": "сентаво",
    "MYR": "сен",
    "MZN": "сентаво",
    "NAD": "цент",
    "NGN": "кобо",
    "NIO": "сентаво",
    "NPR": "пайса",
    "NZD": "цент",
    "OMR": "байза",
    "PAB": "сентесимо",
    "PEN": "сентимо",
    "PGK": "тойя",
    "PHP": "сентимо",
    "PKR": "пайса",
    "PLN": "грош",
    "PYG": "сентимо",
    "QAR": "дирхам",
    "RON": "бань",
    "RSD": "пара",
    "RWF": "cантим",
    "SAR": "халал",
    "SBD": "цент",
    "SCR": "цент",
    "SDG": "пиастр",
    "SHP": "пенс",
    "SLL": "цент",
    "SOS": "цент",
    "SRD": "цент",
    "SSP": "пиастр",
    "STM": "сентимо",
    "SVC": "сентаво",
    "SYP": "пиастр",
    "SZL": "цент",
    "THB": "сатанг",
    "TJS": "дирам",
    "TMT": "тенге",
    "TOP": "сенити",
    "TTD": "цент",
    "TWD": "фынь",
    "TZS": "цент",
    "UGX": "цент",
    "UYU": "сентесимо",
    "UZS": "тийин",
    "VES": "сентимо",
    "VND": "су",
    "WST": "сене",
    "XAF": "сантим",
    "XCD": "цент",
    "XOF": "сантим",
    "XPF": "сантим",
    "YER": "филс",
    "ZAR": "цент",
    "ZMW": "нгве",
    "ZWL": "цент"
}


class Money2Text:
    def __init__(self, morph=None):
        self.morph = pymorphy2.MorphAnalyzer() if morph is None else morph

    def for_special_cases(self, whole, frac, whole_name, frac_name):
        right_whole_name = self.right_text_spelling_with_number(whole, whole_name)
        right_frac_name = self.right_text_spelling_with_number(frac, frac_name)
        return "{:.0f} {} {:.0f} {}".format(whole, right_whole_name, frac, right_frac_name)

    def right_text_spelling_with_number(self, number, text):
        agreed = []
        for token in text.split():
            parse_result = self.morph.parse(token)
            for option in parse_result:
                if option.normal_form == token:
                    word = option
                    break
            else:
                word = parse_result[0]

            agreed_Parse_obj = word.make_agree_with_number(number)
            agreed.append(restore_capitalization(agreed_Parse_obj.word, token) if agreed_Parse_obj else token)
        return " ".join(agreed)

    def _right_word_spelling_with_number(self, number, word):
        butyavka = self.morph.parse(word)[0]
        return butyavka.make_agree_with_number(number).word

    def __call__(self, amount, currency):
        precision = 2
        currency = currency.upper()
        amount = round(amount, precision)
        if currency not in CURRENCY_CODE_2_NAME_MAP:
            return '{} {}'.format(round(amount, 2), currency)

        name = CURRENCY_CODE_2_NAME_MAP[currency]

        frac, whole = modf(amount)
        frac = round(frac, precision) * 100
        if currency in CURRENCY_CODE_2_DEMUNITIVE_NAME_MAP.keys():
            if frac != 0:
                frac_name = CURRENCY_CODE_2_DEMUNITIVE_NAME_MAP[currency]
                return self.for_special_cases(whole=whole, whole_name=name, frac=frac, frac_name=frac_name)
            else:
                return '{0:.0f}'.format(amount) + " " + self.right_text_spelling_with_number(amount, name)
        else:
            if frac == 0:
                return '{0:.0f}'.format(amount) + " " + self.right_text_spelling_with_number(amount, name)
            else:
                return '{}'.format(round(amount, 2)) + " " + self.right_text_spelling_with_number(amount, name)



# coding: utf-8
import unittest

from core.message.msg_validator import MessageValidator
from smart_kit.message.smartapp_to_message import SmartAppToMessage
from smart_kit.utils.picklable_mock import PicklableMock


class TestSmartAppToMessage(unittest.TestCase):
    def setUp(self):
        self.command_ = PicklableMock()
        self.request_ = PicklableMock()
        self.message_ = PicklableMock()
        self.command_.payload = {"z": 1}
        self.command_.name = "AnyName"
        self.command_.loader = "json.dumps"
        self.request_.header = "json"
        self.message_.payload = {"q": 0}
        self.message_.incremental_id = 111
        self.message_.session_id = 11
        self.message_.uuid = '1234-5678-9012'
        self.output_json = '{"messageId": 111, "sessionId": 11, "messageName": "AnyName", "payload": {"z": 1}, ' \
                           '"uuid": "1234-5678-9012"}'

    def test_smart_app_to_message_1(self):
        obj = SmartAppToMessage(self.command_, self.message_, self.request_)
        self.assertTrue(obj.command.name == "AnyName")
        self.assertTrue(obj.incoming_message.uuid == '1234-5678-9012')
        self.assertTrue(obj.request.header == "json")
        self.assertTrue(obj.forward_fields == ())
        self.assertTrue(obj.payload == {"z": 1})
        self.assertTrue(obj.as_dict == {
            "messageId": 111,
            "sessionId": 11,
            "messageName": "AnyName",
            "payload": {"z": 1},
            "uuid": '1234-5678-9012'
        })
        self.assertTrue(obj.value == self.output_json)

    def test_smart_app_to_message_2(self):
        obj = SmartAppToMessage(self.command_, self.message_, self.request_, ["t"])
        self.assertTrue(obj.forward_fields == ["t"])
        self.assertTrue(obj.payload == {"z": 1})

    def test_smart_app_to_message_3(self):
        obj = SmartAppToMessage(self.command_, self.message_, self.request_, ["z"])
        self.assertTrue(obj.forward_fields == ["z"])
        self.assertTrue(obj.payload == {"z": 1})

    def test_smart_app_to_message_4(self):
        obj = SmartAppToMessage(self.command_, self.message_, self.request_, ["q"])
        self.assertTrue(obj.forward_fields == ["q"])
        self.assertTrue(obj.payload == {"z": 1, "q": 0})


class PieMessageValidator(MessageValidator):
    def validate(self, message_name: str, payload: dict):
        return 3.14 < payload.get("pi", 0) < 3.15


class TestSmartAppToMessageValidation(unittest.TestCase):
    def test_validation(self):
        command_ = PicklableMock()
        request_ = PicklableMock()
        message_ = PicklableMock()
        command_.payload = {"pi": 3.14159265358979}
        command_.name = "AnyName"
        request_.header = "json"
        message_.payload = {"q": 0}
        message_.incremental_id = 111
        message_.session_id = 11
        message_.uuid = '1234-5678-9012'

        message = SmartAppToMessage(
            command_, message_, request_,
            validators=(PieMessageValidator(),))
        self.assertTrue(message.validate())

        command_.payload["pi"] = 2.7182818284
        message = SmartAppToMessage(
            command_, message_, request_,
            validators=(PieMessageValidator(),))
        self.assertFalse(message.validate())



from typing import Optional, Union, List, Dict, Any

import core.logging.logger_constants as log_const
from core.basic_models.actions.basic_actions import RequirementAction, ChoiceAction, ElseAction
from core.logging.logger_utils import log
from core.model.factory import factory, list_factory
from core.text_preprocessing.base import BaseTextPreprocessingResult
from core.utils.exception_handlers import exc_handler
from scenarios.scenario_models.field.field_filler_description import FieldFillerDescription
from scenarios.user.user_model import User


class RequirementFiller(RequirementAction):
    FIELD_KEY = "filler"

    def __init__(self, items: Dict[str, Any], id: Optional[str] = None):
        super(RequirementFiller, self).__init__(items, id)
        self.internal_item = self.build_internal_item()

    @factory(FieldFillerDescription)
    def build_internal_item(self):
        return self._item

    def on_extract_error(self, text_preprocessing_result, user, params=None):
        log("exc_handler: RequirementFiller failed to extract. Return None. MESSAGE: %(masked_message)s.",
            user, {log_const.KEY_NAME: log_const.HANDLED_EXCEPTION_VALUE, "masked_message": user.message.masked_value},
            level="ERROR", exc_info=True)
        return None

    @exc_handler(on_error_obj_method_name="on_extract_error")
    def extract(self, text_preprocessing_result: BaseTextPreprocessingResult,
                user: User, params: Dict[str, Any] = None) -> Optional[Union[int, float, str, bool, List, Dict]]:
        return self.run(user, text_preprocessing_result, params)


class ChoiceFiller(ChoiceAction):
    FIELD_REQUIREMENT_KEY = "requirement_fillers"
    FIELD_ELSE_KEY = "else_filler"

    def __init__(self, items: Dict[str, Any], id: Optional[str] = None):
        super(ChoiceFiller, self).__init__(items, id)
        self._requirement_items = items[self.FIELD_REQUIREMENT_KEY]
        self._else_item = items.get(self.FIELD_ELSE_KEY)

        self.items = self.build_items()
        self.else_item = self.build_else_item()

    @list_factory(RequirementFiller)
    def build_items(self):
        return self._requirement_items

    @factory(FieldFillerDescription)
    def build_else_item(self):
        return self._else_item

    def on_extract_error(self, text_preprocessing_result, user, params=None):
        log("exc_handler: ChoiceFiller failed to extract. Return None. MESSAGE: %(masked_message)s.",
            user, {log_const.KEY_NAME: log_const.HANDLED_EXCEPTION_VALUE, "masked_message": user.message.masked_value},
            level="ERROR", exc_info=True)
        return None

    @exc_handler(on_error_obj_method_name="on_extract_error")
    def extract(self, text_preprocessing_result: BaseTextPreprocessingResult,
                user: User, params: Dict[str, Any] = None) -> Optional[Union[int, float, str, bool, List, Dict]]:
        return self.run(user, text_preprocessing_result, params)


class ElseFiller(ElseAction):
    FIELD_ITEM_KEY = "filler"
    FIELD_ELSE_KEY = "else_filler"

    def __init__(self, items: Dict[str, Any], id: Optional[str] = None):
        super(ElseFiller, self).__init__(items, id)
        self._item = items[self.FIELD_ITEM_KEY]
        self._else_item = items.get(self.FIELD_ELSE_KEY)

        self.item = self.build_item()
        self.else_item = self.build_else_item()

    @factory(FieldFillerDescription)
    def build_item(self):
        return self._item

    @factory(FieldFillerDescription)
    def build_else_item(self):
        return self._else_item

    def on_extract_error(self, text_preprocessing_result, user, params=None):
        log("exc_handler: ElseFiller failed to extract. Return None. MESSAGE: %(masked_message)s.",
            user, {log_const.KEY_NAME: log_const.HANDLED_EXCEPTION_VALUE, "masked_message": user.message.masked_value},
            level="ERROR", exc_info=True)
        return None

    @exc_handler(on_error_obj_method_name="on_extract_error")
    def extract(self, text_preprocessing_result: BaseTextPreprocessingResult,
                user: User, params: Dict[str, Any] = None) -> Optional[Union[int, float, str, bool, List, Dict]]:
        return self.run(user, text_preprocessing_result, params)



import os
from typing import List

from core.model.registered import Registered
from core.repositories.base_repository import BaseRepository


class BaseConfig:
    def __init__(self, **kwargs):
        self.registered_repositories = Registered()
        self.repositories = []
        self.source = kwargs.get("source")

    def __getitem__(self, key):
        return self.registered_repositories[key].data

    def get(self, key, default=None):
        return self.registered_repositories[key].data if key in self.registered_repositories else default

    @property
    def _subfolder(self):
        raise NotImplementedError

    def subfolder_path(self, filename):
        return os.path.join(self._subfolder, filename)

    def init(self):
        self.init_repositories()

    def init_repositories(self):
        for rep in self.repositories:
            rep.load()
            self.registered_repositories[rep.key] = rep

    def raw(self):
        items = self.registered_repositories
        return {key: items[key].data for key in items}

    def _override_repositories(self, main_rep: List[BaseRepository], new_repo: List[BaseRepository]) -> List[BaseRepository]:
        """
        Метод предназначе для переопределения источников для чтения по умолчанию на
        пользовательские
        :param main_rep: Переопределяемый список репозиториев
        :param new_repo: Список замен
        :return: Переопределённый список репозиториев
        """
        temp_dict = dict()
        for el in main_rep:
            temp_dict[el.key] = el

        for el in new_repo:
            temp_dict[el.key] = el

        result = []
        for key, value in temp_dict.items():
            result.append(value)

        return result



from unittest import TestCase

from scenarios.scenario_models.field.field_filler_description import FirstOrgFiller
from smart_kit.utils.picklable_mock import PicklableMock


class TestFirstOrgFiller(TestCase):
    def setUp(self):
        items = {}
        self.filler = FirstOrgFiller(items)

    def test_1(self):
        expected = "тинькофф"

        text_preprocessing_result = PicklableMock()
        text_preprocessing_result.org_token_values = ["тинькофф"]
        result = self.filler.extract(text_preprocessing_result, None)

        self.assertEqual(expected, result)

    def test_2(self):
        expected = "тинькофф"

        text_preprocessing_result = PicklableMock()
        text_preprocessing_result.org_token_values = ["тинькофф", "втб", "мегафон"]
        result = self.filler.extract(text_preprocessing_result, None)

        self.assertEqual(expected, result)

    def test_3(self):
        text_preprocessing_result = PicklableMock()
        text_preprocessing_result.org_token_values = []

        result = self.filler.extract(text_preprocessing_result, None)

        self.assertIsNone(result)



# coding=utf-8
import datetime
import json
import os
import re

from collections import OrderedDict
from math import isnan, isinf
from typing import Optional
from time import time


def convert_version_to_list_of_int(version):
    if not version or re.search("[0-9]", version) is None:
        return None
    list_of_string = [re.sub("[^0-9]", "", part) for part in version.split(".")]
    list_of_int = list(map(int, list_of_string))
    return list_of_int


def subfolder_wrap(parent):
    def inner(child):
        return os.path.join(parent, child)

    return inner


def get_int(word):
    num = None
    if len(word) >= 8 and word[0] == "+":
        num = None
    else:
        try:
            num = int(word)
            if isnan(num) or isinf(num):
                num = None
        except (ValueError, OverflowError):
            num = None
    return num


def get_number(word):
    word = word.replace(',', '.')
    if "." in word or "e" in word:
        number = convert_to_float(word)
    else:
        number = get_int(word)
    return number


def convert_to_float(word: str) -> Optional[float]:
    try:
        num = float(word)
        if isnan(num) or isinf(num):
            return None
        return num
    except (ValueError, OverflowError):
        return None


def convert_to_int(x):
    ix = int(x)
    if x == ix:
        return ix
    else:
        return x


def one_of_elements_is_in_set(elements_set, set_to_check):
    if not isinstance(elements_set, set):
        elements_set = set(elements_set)
    if not isinstance(set_to_check, set):
        set_to_check = set(set_to_check)
    return elements_set.intersection(set_to_check) != set()


def all_elements_are_in_set(elements_set, set_to_check):
    if not isinstance(elements_set, set):
        elements_set = set(elements_set)
    if not isinstance(set_to_check, set):
        set_to_check = set(set_to_check)
    return len(elements_set.intersection(set_to_check)) == min(len(elements_set), len(set_to_check))


def prepare_from_one_to_many_to_one_to_one_dict(dict_one_to_many):
    dict_one_to_one = {}
    for base, synonyms in dict_one_to_many.items():
        for synonym in synonyms:
            dict_one_to_one.update({synonym: base})
    return dict_one_to_one


def merge_numbers(text):
    # пример: квартира стоит 12 345 678 рублей -> квартира стоит 12345678 рублей
    def remove_whitespace(number_match):
        return number_match.group().replace(" ", "")

    regex = re.compile("\d+(\s\d+)+")
    return regex.sub(remove_whitespace, text)


def ordered_dict(data=None):
    return OrderedDict(data)


def list_loader(data, delimiter=None):
    return data.split(delimiter)


def unixtime_to_str(unixtime):
    return datetime.datetime.fromtimestamp(int(unixtime)).strftime('%Y-%m-%d %H:%M:%S')


class HashableDict(dict):
    def __hash__(self):
        return hash(frozenset(self.items()))


def ordered_loader(coded):
    return json.loads(coded, object_pairs_hook=OrderedDict)


def current_time_ms():
    return int(round(time() * 1000))


def time_check(begin_time, reject_timeout):
    return current_time_ms() - begin_time <= reject_timeout if begin_time is not None else True


def deep_update_dict(original, update):
    """
    Recursively updates a original by update.
    Subdict's won't be overwritten but also updated.
    """
    for key, value in original.items():
        if key not in update:
            update[key] = value
        elif isinstance(value, dict):
            deep_update_dict(value, update[key])
    return update



import copy
import time
from functools import cached_property
from typing import Optional, Dict, Any, Union, List

from core.basic_models.actions.basic_actions import Action
from core.basic_models.actions.command import Command
from core.basic_models.actions.string_actions import StringAction
from core.basic_models.parametrizers.parametrizer import BasicParametrizer
from core.basic_models.requirement.basic_requirements import Requirement
from core.configs.global_constants import CALLBACK_ID_HEADER, SAVED_BEHAVIOR_PARAMS_FIELDS
from core.logging.logger_utils import log
from core.model.factory import factory, list_factory
from core.text_preprocessing.base import BaseTextPreprocessingResult
from core.text_preprocessing.preprocessing_result import TextPreprocessingResult
from core.unified_template.unified_template import UnifiedTemplate
from core.utils.pickle_copy import pickle_deepcopy

import scenarios.logging.logger_constants as log_const
from scenarios.actions.action_params_names import TO_MESSAGE_NAME, TO_MESSAGE_PARAMS, SAVED_MESSAGES, \
    REQUEST_FIELD
from scenarios.user.parametrizer import Parametrizer
from scenarios.user.user_model import User
from scenarios.scenario_models.history import Event
from smart_kit.names.action_params_names import SEND_TIMESTAMP


class ClearFormAction(Action):
    version: Optional[int]
    parametrizer: BasicParametrizer
    form: str

    def __init__(self, items: Dict[str, Any], id: Optional[str] = None):
        super().__init__(items, id)
        self.form = items["form"]

    def run(self, user: User, text_preprocessing_result: BaseTextPreprocessingResult,
            params: Optional[Dict[str, Union[str, float, int]]] = None) -> List[Command]:
        commands = super().run(user, text_preprocessing_result, params)
        user.forms.remove_item(self.form)
        return commands


class ClearInnerFormAction(ClearFormAction):
    version: Optional[int]
    parametrizer: BasicParametrizer
    form: str
    inner_form: str

    def __init__(self, items: Dict[str, Any], id: Optional[str] = None):
        super().__init__(items, id)
        self.inner_form = items["inner_form"]

    def run(self, user: User, text_preprocessing_result: BaseTextPreprocessingResult,
            params: Optional[Dict[str, Union[str, float, int]]] = None) -> List[Command]:
        commands = super().run(user, text_preprocessing_result, params)
        form = user.forms[self.form]
        if form:
            form.forms.remove_item(self.inner_form)
        return commands


class RemoveFormFieldAction(Action):
    version: Optional[int]
    parametrizer: BasicParametrizer
    form: str
    field: str

    def __init__(self, items: Dict[str, Any], id: Optional[str] = None):
        super().__init__(items, id)
        self.form = items["form"]
        self.field = items["field"]

    def run(self, user: User, text_preprocessing_result: BaseTextPreprocessingResult,
            params: Optional[Dict[str, Union[str, float, int]]] = None) -> List[Command]:
        commands = super().run(user, text_preprocessing_result, params)
        form = user.forms[self.form]
        form.fields.remove_item(self.field)
        return commands


class RemoveCompositeFormFieldAction(RemoveFormFieldAction):
    version: Optional[int]
    parametrizer: BasicParametrizer
    form: str
    field: str
    inner_form: str

    def __init__(self, items: Dict[str, Any], id: Optional[str] = None):
        super().__init__(items, id)
        self.inner_form = items["inner_form"]

    def run(self, user: User, text_preprocessing_result: BaseTextPreprocessingResult,
            params: Optional[Dict[str, Union[str, float, int]]] = None) -> List[Command]:
        commands = super().run(user, text_preprocessing_result, params)
        form = user.forms[self.form]
        inner_form = form.forms[self.inner_form]
        inner_form.fields.remove_item(self.field)
        return commands


class BreakScenarioAction(Action):
    scenario_id: Optional[str]

    def __init__(self, items: Dict[str, Any], id: Optional[str] = None):
        super().__init__(items, id)
        self.scenario_id = items.get("scenario_id")

    def run(self, user: User, text_preprocessing_result: BaseTextPreprocessingResult,
            params: Optional[Dict[str, Union[str, float, int]]] = None) -> List[Command]:
        commands = super().run(user, text_preprocessing_result, params)
        scenario_id = self.scenario_id if self.scenario_id is not None else user.last_scenarios.last_scenario_name
        user.scenario_models[scenario_id].set_break()
        return commands


class SaveBehaviorAction(Action):
    version: Optional[int]
    parametrizer: BasicParametrizer
    behavior: str
    check_scenario: bool

    def __init__(self, items: Dict[str, Any], id: Optional[str] = None):
        super().__init__(items, id)
        self.behavior = items["behavior"]
        self.check_scenario = items.get("check_scenario", True)

    def run(self, user: User, text_preprocessing_result: BaseTextPreprocessingResult,
            params: Optional[Dict[str, Union[str, float, int]]] = None) -> List[Command]:
        commands = super().run(user, text_preprocessing_result, params)
        scenario_id = None
        if self.check_scenario:
            scenario_id = user.last_scenarios.last_scenario_name
        user.behaviors.add(user.message.generate_new_callback_id(), self.behavior, scenario_id,
                           text_preprocessing_result.raw, action_params=pickle_deepcopy(params))
        return commands


class BasicSelfServiceActionWithState(StringAction):
    version: Optional[int]
    parametrizer: BasicParametrizer
    behavior: str
    command_action: Action

    def __init__(self, items: Dict[str, Any], id: Optional[str] = None):
        self.behavior = items["behavior"]
        self._command_action: Dict[str, Any] = items["command_action"]
        self._check_scenario: bool = items.get("check_scenario", True)
        super().__init__(self._command_action, id)

    @cached_property
    def behavior_action(self) -> SaveBehaviorAction:
        return SaveBehaviorAction({"behavior": self.behavior, "check_scenario": self._check_scenario})

    @cached_property
    def command_action(self) -> StringAction:
        return StringAction(self._command_action)

    def _check(self, user):
        return not user.behaviors.check_got_saved_id(self.behavior_action.behavior)

    def _run(self, user, text_preprocessing_result, params=None) -> List[Command]:
        self.behavior_action.run(user, text_preprocessing_result, params)
        command_action_result = self.command_action.run(user, text_preprocessing_result, params)
        return command_action_result

    def run(self, user: User, text_preprocessing_result: BaseTextPreprocessingResult,
            params: Optional[Dict[str, Union[str, float, int]]] = None) -> List[Command]:
        commands = super().run(user, text_preprocessing_result, params)
        if self._check(user):
            commands.extend(self._run(user, text_preprocessing_result, params))
            return commands


class FillFieldAction(Action):
    version: Optional[int]
    parametrizer: BasicParametrizer
    form: str
    field: str
    data_path: Union[str, Dict[str, Any]]

    def __init__(self, items: Dict[str, Any], id: Optional[str] = None):
        super().__init__(items, id)
        self.form = items["form"]
        self.field: str = items["field"]
        data_path = items["data_path"]
        self.template: UnifiedTemplate = UnifiedTemplate(data_path)

    def _fill(self, user, data):
        if data is not None:
            user.forms[self.form].fields[self.field].set_available()
            user.forms[self.form].fields[self.field].fill(data)

    def _get_data(self, params):
        return self.template.render(params)

    def run(self, user: User, text_preprocessing_result: BaseTextPreprocessingResult,
            params: Optional[Dict[str, Union[str, float, int]]] = None) -> List[Command]:
        commands = super().run(user, text_preprocessing_result, params)
        params = user.parametrizer.collect(text_preprocessing_result)
        data = self._get_data(params)
        self._fill(user, data)
        return commands


class CompositeFillFieldAction(FillFieldAction):
    version: Optional[int]
    parametrizer: BasicParametrizer
    internal_form: str

    def __init__(self, items: Dict[str, Any], id: Optional[str] = None):
        super().__init__(items, id)
        self.internal_form = items["internal_form"]

    def _fill(self, user, data):
        if data is not None:
            user.forms[self.form].forms[self.internal_form].fields[self.field].set_available()
            user.forms[self.form].forms[self.internal_form].fields[self.field].fill(data)


class RunScenarioAction(Action):
    version: Optional[int]
    parametrizer: BasicParametrizer

    def __init__(self, items: Dict[str, Any], id: Optional[str] = None):
        super().__init__(items, id)
        self.scenario: UnifiedTemplate = UnifiedTemplate(items["scenario"])

    def run(self, user: User, text_preprocessing_result: BaseTextPreprocessingResult,
            params: Optional[Dict[str, Union[str, float, int]]] = None) -> List[Command]:
        commands = super().run(user, text_preprocessing_result, params)
        if params is None:
            params = user.parametrizer.collect(text_preprocessing_result)
        else:
            params.update(user.parametrizer.collect(text_preprocessing_result))
        scenario_id = self.scenario.render(params)
        scenario = user.descriptions["scenarios"].get(scenario_id)
        if scenario:
            commands.extend(scenario.run(text_preprocessing_result, user, params))
        return commands


class RunLastScenarioAction(Action):
    def run(self, user: User, text_preprocessing_result: BaseTextPreprocessingResult,
            params: Optional[Dict[str, Union[str, float, int]]] = None) -> List[Command]:
        commands = super().run(user, text_preprocessing_result, params)
        last_scenario_id = user.last_scenarios.last_scenario_name
        scenario = user.descriptions["scenarios"].get(last_scenario_id)
        if scenario:
            commands.extend(scenario.run(text_preprocessing_result, user, params))
        return commands


class ChoiceScenarioAction(Action):
    FIELD_SCENARIOS_KEY = "scenarios"
    FIELD_ELSE_KEY = "else_action"
    FIELD_REQUIREMENT_KEY = "requirement"

    def __init__(self, items: Dict[str, Any], id: Optional[str] = None) -> None:
        super().__init__(items, id)
        self._else_item = items.get(self.FIELD_ELSE_KEY)
        self._scenarios = items[self.FIELD_SCENARIOS_KEY]
        self._requirements = [scenario.pop(self.FIELD_REQUIREMENT_KEY) for scenario in self._scenarios]

        self.requirement_items = self.build_requirement_items()

        if self._else_item:
            self.else_item = self.build_else_item()
        else:
            self.else_item = None

    @list_factory(Requirement)
    def build_requirement_items(self):
        return self._requirements

    @factory(Action)
    def build_else_item(self):
        return self._else_item

    def run(self, user: User, text_preprocessing_result: BaseTextPreprocessingResult,
            params: Optional[Dict[str, Union[str, float, int]]] = None) -> List[Command]:
        commands = super().run(user, text_preprocessing_result, params)
        choice_is_made = False

        for scenario, requirement in zip(self._scenarios, self.requirement_items):
            check_res = requirement.check(text_preprocessing_result, user, params)
            if check_res:
                commands.extend(RunScenarioAction(items=scenario).run(user, text_preprocessing_result, params))
                choice_is_made = True
                break

        if not choice_is_made and self._else_item:
            commands.extend(self.else_item.run(user, text_preprocessing_result, params))

        return commands


def clear_scenario(user, scenario_id):
    scenario = user.descriptions["scenarios"][scenario_id]
    user.last_scenarios.delete(scenario_id)
    user.forms.remove_item(scenario.form_type)


class ClearCurrentScenarioAction(Action):
    def run(self, user: User, text_preprocessing_result: BaseTextPreprocessingResult,
            params: Optional[Dict[str, Union[str, float, int]]] = None) -> List[Command]:
        commands = super().run(user, text_preprocessing_result, params)
        last_scenario_id = user.last_scenarios.last_scenario_name
        if last_scenario_id:
            clear_scenario(user, last_scenario_id)
        return commands


class ClearAllScenariosAction(Action):
    def run(self, user: User, text_preprocessing_result: BaseTextPreprocessingResult,
            params: Optional[Dict[str, Union[str, float, int]]] = None) -> List[Command]:
        commands = super().run(user, text_preprocessing_result, params)
        user.last_scenarios.clear_all()
        return commands


class ClearScenarioByIdAction(Action):
    version: Optional[int]
    parametrizer: BasicParametrizer
    scenario_id: str

    def __init__(self, items: Dict[str, Any], id: Optional[str] = None):
        super().__init__(items, id)
        self.scenario_id = items.get("scenario_id")

    def run(self, user: User, text_preprocessing_result: BaseTextPreprocessingResult,
            params: Optional[Dict[str, Union[str, float, int]]] = None) -> List[Command]:
        commands = super().run(user, text_preprocessing_result, params)
        if self.scenario_id:
            clear_scenario(user, self.scenario_id)
        return commands


class ClearCurrentScenarioFormAction(Action):
    def __init__(self, items: Dict[str, Any], id: Optional[str] = None):
        super().__init__(items, id)

    def run(self, user: User, text_preprocessing_result: BaseTextPreprocessingResult,
            params: Optional[Dict[str, Union[str, float, int]]] = None) -> List[Command]:
        commands = super().run(user, text_preprocessing_result, params)
        last_scenario_id = user.last_scenarios.last_scenario_name
        if last_scenario_id:
            user.forms.clear_form(last_scenario_id)
        return commands


class ResetCurrentNodeAction(Action):
    def __init__(self, items: Dict[str, Any], id: Optional[str] = None):
        super().__init__(items, id)
        self.node_id = items.get('node_id', None)

    def run(self, user: User, text_preprocessing_result: BaseTextPreprocessingResult,
            params: Optional[Dict[str, Union[str, float, int]]] = None) -> List[Command]:
        commands = super().run(user, text_preprocessing_result, params)
        last_scenario_id = user.last_scenarios.last_scenario_name
        if last_scenario_id:
            user.scenario_models[last_scenario_id].current_node = self.node_id
        return commands


class AddHistoryEventAction(Action):
    results: UnifiedTemplate
    event_type: UnifiedTemplate
    event_content: Dict[str, UnifiedTemplate]

    def __init__(self, items: Dict[str, Any], id: Optional[str] = None):
        super().__init__(items, id)
        self.results = UnifiedTemplate(items.get("results"))
        self.event_type = UnifiedTemplate(items.get("event_type"))
        self.event_content = items.get("event_content")
        if self.event_content:
            for k, v in self.event_content.items():
                self.event_content[k] = UnifiedTemplate(v)

    def run(self, user: User, text_preprocessing_result: BaseTextPreprocessingResult,
            params: Optional[Dict[str, Union[str, float, int]]] = None) -> List[Command]:
        commands = super().run(user, text_preprocessing_result, params)
        last_scenario_id = user.last_scenarios.last_scenario_name
        scenario = user.descriptions["scenarios"].get(last_scenario_id)
        if scenario:

            params = user.parametrizer.collect(text_preprocessing_result)

            if self.event_content:
                for key, template in self.event_content.items():
                    self.event_content[key] = template.render(params)
            self.event_type = self.event_type.render(params)
            self.results = self.results.render(params)

            event = Event(
                type=self.event_type,
                scenario=scenario.id,
                scenario_version=scenario.version,
                result=self.results,
                content=self.event_content
            )
            user.history.add_event(event)
        return commands


class EmptyAction(Action):
    def run(self, user: User, text_preprocessing_result: BaseTextPreprocessingResult,
            params: Optional[Dict[str, Union[str, float, int]]] = None) -> List[Command]:
        commands = super().run(user, text_preprocessing_result, params)
        log("%(class_name)s.run: action do nothing.",
            params={log_const.KEY_NAME: "empty_action", "class_name": self.__class__.__name__}, user=user)
        return commands


class RunScenarioByProjectNameAction(Action):
    def run(self, user: User, text_preprocessing_result: TextPreprocessingResult,
            params: Optional[Dict[str, Union[str, float, int]]] = None) -> List[Command]:
        commands = super().run(user, text_preprocessing_result, params)
        scenario_id = user.message.project_name
        scenario = user.descriptions["scenarios"].get(scenario_id)
        if scenario:
            commands.extend(scenario.run(text_preprocessing_result, user, params))
        else:
            log("%(class_name)s warning: %(scenario_id)s isn't exist",
                params={log_const.KEY_NAME: "warning_in_RunScenarioByProjectNameAction",
                        "class_name": self.__class__.__name__, "scenario_id": scenario_id},
                user=user, level="WARNING")
        return commands


class ProcessBehaviorAction(Action):
    def run(self, user: User, text_preprocessing_result: BaseTextPreprocessingResult,
            params: Optional[Dict[str, Union[str, float, int]]] = None) -> List[Command]:
        commands = super().run(user, text_preprocessing_result, params)
        callback_id = user.message.callback_id

        log("%(class_name)s.run: got callback_id %(callback_id)s.",
            params={log_const.KEY_NAME: "process_behavior_action",
                    "class_name": self.__class__.__name__,
                    log_const.BEHAVIOR_CALLBACK_ID_VALUE: callback_id}, user=user)

        if not user.behaviors.has_callback(callback_id):
            log("%(class_name)s.run: user.behaviors has no callback %(callback_id)s.",
                params={log_const.KEY_NAME: "process_behavior_action_warning",
                        "class_name": self.__class__.__name__,
                        log_const.BEHAVIOR_CALLBACK_ID_VALUE: callback_id}, level="WARNING", user=user)
            return commands

        if user.message.payload:
            commands.extend(user.behaviors.success(callback_id))
        else:
            commands.extend(user.behaviors.fail(callback_id))
        return commands


class SelfServiceActionWithState(BasicSelfServiceActionWithState):
    version: Optional[int]
    parametrizer: Parametrizer
    behavior: str
    command_action: Action

    def __init__(self, items: Dict[str, Any], id: Optional[str] = None):
        super().__init__(items, id)
        self.save_params_template_data = self._get_template_tree(items.get("save_params") or {})
        self.rewrite_saved_messages = items.get("rewrite_saved_messages", False)
        self._check_scenario: bool = items.get("check_scenario", True)

    def _run(self, user, text_preprocessing_result, params=None) -> List[Command]:

        action_params = copy.copy(params or {})

        command_params = dict()
        collected = user.parametrizer.collect(text_preprocessing_result, filter_params={"command": self.command})
        action_params.update(collected)

        scenario = None
        if self._check_scenario:
            scenario = user.last_scenarios.last_scenario_name

        for key, value in self.nodes.items():
            rendered = self._get_rendered_tree(value, action_params, self.no_empty_nodes)
            if rendered != "" or not self.no_empty_nodes:
                command_params[key] = rendered

        callback_id = user.message.generate_new_callback_id()
        request_data = copy.copy(self.request_data or {})
        request_data.update(self._get_extra_request_data(user, params, callback_id))

        save_params = self._get_save_params(user, action_params, command_params)
        self._save_behavior(callback_id, user, scenario, text_preprocessing_result, save_params)

        commands = [Command(self.command, command_params, self.id, request_type=self.request_type,
                            request_data=request_data)]
        return commands

    def _get_extra_request_data(self, user, params, callback_id):
        extra_request_data = {}
        extra_request_data[CALLBACK_ID_HEADER] = callback_id
        return extra_request_data

    def _save_behavior(self, callback_id, user, scenario, text_preprocessing_result, save_params):
        user.behaviors.add(
            callback_id,
            self.behavior,
            scenario,
            text_preprocessing_result.raw,
            save_params,
        )

    def _get_save_params(self, user, action_params, command_action_params):
        save_params = self._get_rendered_tree_recursive(self.save_params_template_data, action_params)
        keys_to_pop = []
        for key in save_params.keys():
            if key not in SAVED_BEHAVIOR_PARAMS_FIELDS:
                keys_to_pop.append(key)
        for key in keys_to_pop:
            save_params.pop(key)
        save_params.update({SAVED_MESSAGES: action_params.get(SAVED_MESSAGES, {})})
        save_params.update({REQUEST_FIELD: action_params.get(REQUEST_FIELD, {})})
        save_params.update({SEND_TIMESTAMP: time.time()})

        if user.settings["template_settings"].get("self_service_with_state_save_messages", True):
            saved_messages = save_params[SAVED_MESSAGES]
            if user.message.message_name not in saved_messages or self.rewrite_saved_messages:
                saved_messages[user.message.type] = user.message.payload

        save_params.update({TO_MESSAGE_PARAMS: command_action_params})
        save_params.update({TO_MESSAGE_NAME: self.command})
        return save_params



import json
import os
from csv import DictWriter, QUOTE_MINIMAL
from functools import cached_property
from typing import AnyStr, Optional, Tuple, Any, Dict, Callable

from core.configs.global_constants import LINK_BEHAVIOR_FLAG
from smart_kit.compatibility.commands import combine_commands
from smart_kit.configs.settings import Settings
from smart_kit.message.smartapp_to_message import SmartAppToMessage
from smart_kit.models.smartapp_model import SmartAppModel
from smart_kit.request.kafka_request import SmartKitKafkaRequest
from smart_kit.testing.utils import Environment
from smart_kit.utils.diff import partial_diff


def run_testfile(path: AnyStr, file: AnyStr, app_model: SmartAppModel, settings: Settings, user_cls: type,
                 parametrizer_cls: type, from_msg_cls: type, test_case_cls: type, storaged_predefined_fields: Dict[str, Any],
                 csv_file_callback: Optional[Callable[[str], Callable[[Any], None]]] = None,
                 interactive: bool = False) -> Tuple[int, int]:
    test_file_path = os.path.join(path, file)
    if not os.path.isfile(test_file_path) or not test_file_path.endswith('.json'):
        raise FileNotFoundError
    with open(test_file_path, "r") as test_file:
        json_obj = json.load(test_file)
    success = 0
    for test_case in json_obj:
        test_params = json_obj[test_case]
        if isinstance(test_params, list):
            test_params = {"messages": test_params, "user": {}}
        print(f"[+] Processing test case {test_case} from {test_file_path}")
        if csv_file_callback:
            csv_case_callback = csv_file_callback(test_case)
        else:
            csv_case_callback = None
        if test_case_cls(
                app_model,
                settings,
                user_cls,
                parametrizer_cls,
                from_msg_cls,
                **test_params,
                storaged_predefined_fields=storaged_predefined_fields,
                interactive=interactive,
                csv_case_callback=csv_case_callback,
        ).run():
            print(f"[+] {test_case} OK")
            success += 1
    print(f"[+] {file} {success}/{len(json_obj)}")
    return len(json_obj), success


class TestSuite:
    def __init__(self, path: AnyStr, app_config: Any, predefined_fields_storage: AnyStr, make_csv: bool):
        self.path = path
        self.app_config = app_config

        self.csv_callback = None
        if make_csv:
            field_names = ['file', 'test_case', 'success', 'diff']
            results_csv_writer = DictWriter(
                open(os.path.join(path, f'tests_results.csv'), 'wt'),
                fieldnames=field_names,
                quoting=QUOTE_MINIMAL
            )
            results_csv_writer.writeheader()

            def __csv_file_callback(file):
                def __csv_test_case_callback(test_case_name):
                    def __write_csv_line(diff):
                        results_csv_writer.writerow(dict(zip(
                            field_names,
                            [file, test_case_name, 0 if diff else 1, diff.serialize()]
                        )))
                    return __write_csv_line
                return __csv_test_case_callback
            self.csv_callback = __csv_file_callback

        with open(predefined_fields_storage, "r") as f:
            self.storaged_predefined_fields = json.load(f)

    @cached_property
    def app_model(self):
        return self.app_config.MODEL(
            self.resources, self.app_config.DIALOGUE_MANAGER, custom_settings=self.settings,
            app_name=self.app_config.APP_NAME
        )

    @cached_property
    def resources(self):
        source = self.settings.get_source()

        return self.app_config.RESOURCES(source, self.app_config.REFERENCES_PATH, self.settings)

    @cached_property
    def settings(self):
        return self.app_config.SETTINGS(config_path=self.app_config.CONFIGS_PATH,
                                        secret_path=self.app_config.SECRET_PATH,
                                        references_path=self.app_config.REFERENCES_PATH,
                                        app_name=self.app_config.APP_NAME)

    def run(self):
        total = 0
        total_success = 0
        for path, dirs, files in os.walk(self.path):
            for file in files:
                if not file.endswith(".json"):
                    continue
                if self.csv_callback:
                    csv_file_callback = self.csv_callback(file)
                else:
                    csv_file_callback = None
                file_total, file_success = run_testfile(
                    path,
                    file,
                    self.app_model,
                    self.settings,
                    self.app_config.USER,
                    self.app_config.PARAMETRIZER,
                    self.app_config.FROM_MSG,
                    self.app_config.TEST_CASE,
                    self.storaged_predefined_fields,
                    csv_file_callback
                )
                total += file_total
                total_success += file_success

        print(f"[+] Total: {total_success}/{total}")


class TestCase:
    def __init__(self, app_model: SmartAppModel, settings: Settings, user_cls: type, parametrizer_cls: type,
                 from_msg_cls: type, messages: dict, storaged_predefined_fields: Dict[str, Any], interactive: bool,
                 csv_case_callback: Optional[Callable[[Any], None]] = None, user: Optional[dict] = None):
        self.messages = messages
        self.user_state = json.dumps(user)
        self.interactive = interactive

        self.app_model = app_model
        self.settings = settings
        self.storaged_predefined_fields = storaged_predefined_fields
        self.csv_case_callback = csv_case_callback

        self.__parametrizer_cls = parametrizer_cls
        self.__user_cls = user_cls
        self.__from_msg_cls = from_msg_cls

    def run(self) -> bool:
        success = True

        app_callback_id = None
        for index, message_ in enumerate(self.messages):
            print('Шаг', index)
            if index and self.interactive:
                print("Нажмите ENTER, чтобы продолжить...")
                input()

            request = message_["request"]
            response = message_["response"]

            # Если использован флаг linkPreviousByCallbackId и после предыдущего сообщения был сохранен app_callback_id,
            # сообщению добавляются заголовки. Таким образом, сработает behavior, созданный предыдущим запросом
            if message_.get(LINK_BEHAVIOR_FLAG) and app_callback_id:
                headers = [(self.__from_msg_cls.CALLBACK_ID_HEADER_NAME, app_callback_id.encode())]
            else:
                headers = [('kafka_correlationId', 'test_123')]
            message = self.create_message(request, headers=headers)

            user = self.__user_cls(
                id=message.uid, message=message, db_data=self.user_state, settings=self.settings,
                descriptions=self.app_model.scenario_descriptions,
                parametrizer_cls=self.__parametrizer_cls
            )

            self.post_setup_user(user)

            commands = self.app_model.answer(message, user) or []

            answers = self._generate_answers(
                user=user, commands=commands, message=message
            )

            predefined_fields_resp = response.get("predefined_fields")
            if predefined_fields_resp:
                response = self.handle_predefined_fields_response(predefined_fields_resp, response)
            expected_answers = response["messages"]
            expected_user = response["user"]

            if len(commands) != len(response["messages"]):
                print(
                    f"[!] Expected quantity of messages differ from received.\n"
                    f" Expected: {len(response['messages'])}. Actual: {len(answers)}."
                )
                success = False
                continue

            app_callback_id = None
            for actual, expected in zip(answers, expected_answers):
                actual_value = actual.as_dict
                diff = partial_diff(expected, actual_value)
                if diff:
                    success = False
                    print(diff)
                if self.csv_case_callback:
                    self.csv_case_callback(diff)
                # Последний app_callback_id в answers, используется в заголовках следующего сообщения
                app_callback_id = actual.request.values.get(self.__from_msg_cls.CALLBACK_ID_HEADER_NAME, app_callback_id)

            user_diff = partial_diff(expected_user, user.raw)
            if user_diff:
                success = False
                print(user_diff)
            self.user_state = user.raw_str
        return success

    def _generate_answers(self, user, commands, message):
        answers = []
        commands = commands or []

        commands = combine_commands(commands, user)

        for command in commands:
            request = SmartKitKafkaRequest(id=None, items=command.request_data)
            answer = SmartAppToMessage(command=command, message=message, request=request)
            answers.append(answer)
        return answers

    def create_message(self, data, headers=None):
        defaults = Environment().as_dict

        predefined_fields = data.get("predefined_fields")
        is_payload_field = data.get("payload")
        if predefined_fields:
            predefined_fields_data = self.storaged_predefined_fields[predefined_fields]
            if not is_payload_field and not predefined_fields_data.get("payload"):
                predefined_fields_data = {"payload": predefined_fields_data}
            if is_payload_field and predefined_fields_data.get("payload"):
                raise Exception("Payload field is in test case and in predefined_fields object, check it!")
            defaults.update(predefined_fields_data)
            del data["predefined_fields"]

        message = data.get("message")
        if message:
            defaults["payload"].update({"message": message})

        defaults.update(data)
        return self.__from_msg_cls(defaults, headers=headers)

    def handle_predefined_fields_response(self, predefined_fields_resp, response):
        predefined_fields_resp_data = self.storaged_predefined_fields[predefined_fields_resp]
        response.update(predefined_fields_resp_data)
        del response["predefined_fields"]

        pronounce_texts = response.get("pronounce_texts", [])
        if pronounce_texts:
            for text, msg_dict in zip(pronounce_texts, response["messages"]):
                msg_dict["payload"]["pronounceText"] = text
            del response["pronounce_texts"]

        return response

    def post_setup_user(self, user):
        pass



# coding: utf-8
import unittest
from unittest.mock import Mock, MagicMock

from smart_kit.handlers import handle_respond
from smart_kit.utils.picklable_mock import PicklableMock, PicklableMagicMock


class HandlerTest4(unittest.TestCase):
    def setUp(self):
        self.app_name = "TestAppName"
        self.test_user1 = Mock('user')
        self.test_user1.id = '123-345-678'  # пусть чему-то равняется
        self.test_user1.descriptions = {}
        self.test_user1.message = Mock('message')
        self.test_user1.message.callback_id = 11  # пусть чему-то равняется
        self.test_user1.message.incremental_id = 22  # пусть чему-то равняется
        self.test_user1.message.logging_uuid = '321-654-987'  # пусть чему-то равняется
        self.test_user1.message.message_name = "TestMessageName"
        self.test_user1.message.channel = "test_channel"
        self.test_user1.message.device = PicklableMock()
        self.test_user1.message.device.surface = "test_surface"
        self.test_user1.message.app_info = {}
        self.test_user1.behaviors = PicklableMagicMock()

        self.test_action = Mock('action')
        self.test_action.run = lambda x, y, z: [10]  # пусть что то возвращает
        self.test_user2 = MagicMock('user')
        self.test_user2.id = '123-345-678'  # пусть чему-то равняется
        self.test_user2.descriptions = {'external_actions': {'any action name': self.test_action}}
        self.test_user2.message = Mock('message')
        self.test_user2.message.callback_id = 11  # пусть чему-то равняется
        self.test_user2.message.incremental_id = 22  # пусть чему-то равняется
        self.test_user2.message.logging_uuid = '321-654-987'  # пусть чему-то равняется
        self.test_user2.message.message_name = "TestMessageName"
        self.test_user2.message.channel = "test_channel"
        self.test_user2.message.device = PicklableMock()
        self.test_user2.message.device.surface = "test_surface"
        self.test_user2.message.app_info = {}
        self.callback11_action_params = PicklableMagicMock()
        self.test_user2.behaviors = PicklableMock()
        self.test_user2.behaviors.get_callback_action_params = MagicMock(return_value=self.callback11_action_params)

        self.test_payload = {'message': {1: 1}}

    def test_handler_respond_init(self):
        obj1 = handle_respond.HandlerRespond(app_name=self.app_name)
        obj2 = handle_respond.HandlerRespond(self.app_name, "any action name")
        self.assertIsNotNone(obj1.KAFKA_KEY)
        self.assertIsNone(obj1._action_name)
        self.assertIsNotNone(obj2._action_name)

    def test_handler_respond_get_action_name(self):
        obj1 = handle_respond.HandlerRespond(app_name=self.app_name)
        obj2 = handle_respond.HandlerRespond(self.app_name, "any action name")
        self.assertIsNone(obj1.get_action_name(self.test_payload, self.test_user1))
        self.assertIsNotNone(obj2.get_action_name(self.test_payload, self.test_user1))

    def test_handler_respond_get_action_params(self):
        obj = handle_respond.HandlerRespond(app_name=self.app_name)
        self.assertTrue(obj.get_action_params(self.test_payload, self.test_user2) == self.callback11_action_params)
        self.assertTrue(obj.get_action_params("any data", self.test_user2) == self.callback11_action_params)
        self.assertTrue(obj.get_action_params(None, self.test_user2) == self.callback11_action_params)

    def test_handler_respond_run(self):
        self.assertIsNotNone(handle_respond.TextPreprocessingResult.from_payload(self.test_payload))
        self.assertIsNotNone(handle_respond.log_const.KEY_NAME)
        self.assertIsNotNone(handle_respond.log_const.NORMALIZED_TEXT_VALUE)
        self.assertIsNotNone(handle_respond.TextPreprocessingResult.from_payload(self.test_payload).raw)
        obj1 = handle_respond.HandlerRespond(app_name=self.app_name)
        obj2 = handle_respond.HandlerRespond(self.app_name, "any action name")
        with self.assertRaises(KeyError):
            obj1.run(self.test_payload, self.test_user1)
        self.assertTrue(obj2.run(self.test_payload, self.test_user2) == [10])



# coding: utf-8
from functools import cached_property

from core.basic_models.actions.basic_actions import Action
from core.basic_models.requirement.basic_requirements import Requirement
from core.model.factory import factory, list_factory, build_factory
from core.model.registered import Registered
from scenarios.scenario_models.field.field_filler_description import FieldFillerDescription
from scenarios.scenario_models.field_validator.field_validator import FieldValidator

field_descriptions = Registered()

field_description_factory = build_factory(field_descriptions)


class BasicFieldDescription:
    DEFAULT_AVAILABLE_VALUE = True

    def __init__(self, items, id):
        self.id = id
        self.name = id
        self.type = items.get("type")
        self._required = items.get("required", False)
        self._filler = items.get("filler")
        self.default_value = items.get("default_value")
        self.available = items.get("available", self.DEFAULT_AVAILABLE_VALUE)
        self._requests = items.get("requests", [])
        self._on_filled_actions = items.get("on_filled_actions", [])
        self._requirement = items.get("requirement")
        self._field_validator = items.get("field_validator")
        self.need_save_context = items.get("need_save_context", False)
        self.need_load_context = items.get("need_load_context", False)
        self.fill_other = items.get("fill_other", True)
        self._ask_again_requests = items.get("ask_again_questions", [])

    @cached_property
    @list_factory(Action)
    def requests(self):
        return self._requests

    @cached_property
    @list_factory(Action)
    def on_filled_actions(self):
        return self._on_filled_actions

    @property
    def has_requests(self):
        return True if self._requests else False

    @property
    def required(self):
        return self._required

    @cached_property
    @factory(Requirement)
    def requirement(self):
        return self._requirement

    @cached_property
    @factory(FieldFillerDescription)
    def filler(self):
        return self._filler

    @cached_property
    @factory(FieldValidator)
    def field_validator(self):
        return self._field_validator



NUM_TOKEN = "NUM_TOKEN"
TWO_NUM_TOKENS = "{} {}".format(NUM_TOKEN, NUM_TOKEN)
CCY_TOKEN = "CCY_TOKEN"
PHONE_NUMBER_TOKEN = "PHONE_NUMBER_TOKEN"
TIME_DATE_TOKEN = "TIME_DATE_TOKEN"
TIME_YEAR_TOKEN = "TIME_YEAR_TOKEN"
TIME_MONTH_TOKEN = "TIME_MONTH_TOKEN"
CARD_NUMBER_TOKEN = "CARD_NUMBER_TOKEN"
ANIMACY_TOKEN = "ANIMACY_TOKEN"
SENTENCE_ENDPOINT_TOKEN = "SENTENCE_ENDPOINT_TOKEN"
TIME_DAY_TOKEN = "TIME_DAY_TOKEN"
TIME_TIME_TOKEN = "TIME_TIME_TOKEN"
TIME_TEMPORAL_TOKEN = "TIME_TEMPORAL_TOKEN"
NAME_TOKEN = "NAME_TOKEN"
SURNAME_TOKEN = "SURNAME_TOKEN"
PATRONYMIC_TOKEN = "PATRONYMIC_TOKEN"
EMAIL_ADDRESS_TOKEN = "EMAIL_ADDRESS_TOKEN"
URL_TOKEN = "URL_TOKEN"
EMOJI_TOKEN = "EMOJI_TOKEN"
GEO_TOKEN = "GEO_TOKEN"
CATALOG_GEO_TOKEN = "CATALOG_GEO_TOKEN"
ORG_TOKEN = "ORG_TOKEN"
CATALOG_ORG_TOKEN = "CATALOG_ORG_TOKEN"
THUMB_UP_TOKEN = "THUMB_UP_TOKEN"
THUMB_DOWN_TOKEN = "THUMB_DOWN_TOKEN"
SMS_CARDNUM_TOKEN = "SMS_CARDNUM_TOKEN"
SMS_OWNER_CARDNUM_TOKEN = "SMS_OWNER_CARDNUM_TOKEN"
SMS_KEYWORD_TOKEN = "SMS_KEYWORD_TOKEN"
SMS_ALIAS_TOKEN = "SMS_ALIAS_TOKEN"
PERSON_TOKEN = "PERSON_TOKEN"
TIME_DATE_INTERVAL_TOKEN = "TIME_DATE_INTERVAL_TOKEN"
MONEY_TOKEN = "MONEY_TOKEN"
PERIOD_TOKEN = "PERIOD_TOKEN"
RELATIVE_TIME_TOKEN = "RELATIVE_TIME_TOKEN"



from unittest import TestCase

from scenarios.scenario_models.field.field_filler_description import FirstPersonFiller
from smart_kit.utils.picklable_mock import PicklableMock


class TestFirstPersonFiller(TestCase):
    def setUp(self):
        items = {}
        self.filler = FirstPersonFiller(items)

    def test_1(self):
        expected = {"name": "иван"}

        text_preprocessing_result = PicklableMock()
        text_preprocessing_result.person_token_values = [{"name": "иван"}]
        result = self.filler.extract(text_preprocessing_result, None)

        self.assertDictEqual(expected, result)

    def test_2(self):
        expected = {"name": "иван"}

        text_preprocessing_result = PicklableMock()
        text_preprocessing_result.person_token_values = [{"name": "иван"}, {"name": "иван", "patronymic": "иванович"}]
        result = self.filler.extract(text_preprocessing_result, None)

        self.assertDictEqual(expected, result)

    def test_3(self):
        text_preprocessing_result = PicklableMock()
        text_preprocessing_result.person_token_values = []

        result = self.filler.extract(text_preprocessing_result, None)

        self.assertIsNone(result)


from core.model.factory import build_factory
from core.model.registered import Registered
from core.utils.exception_handlers import exc_handler
import timeout_decorator


requests_registered = Registered()
request_factory = build_factory(requests_registered)


class BaseRequest:
    BLOCKING_TIMEOUT = 0.2
    DEFAULT_ENABLED = True

    def __init__(self, items, id=None):
        self.values = items
        self.timeout = items.get("timeout")
        self._timeout_wrap = timeout_decorator.timeout(self.timeout or self.BLOCKING_TIMEOUT)
        self._enabled = items.get("enabled", self.DEFAULT_ENABLED) if items is not None else self.DEFAULT_ENABLED

    @property
    def group_key(self):
        return None

    def check_enabled(self):
        return self._enabled

    def update_empty_items(self, items):
        pass

    @exc_handler()
    def run(self, data, params=None):
        raise NotImplementedError



from typing import List, Any, Dict

from core.basic_models.actions.command import Command
from core.logging.logger_utils import log
from core.text_preprocessing.preprocessing_result import TextPreprocessingResult
import scenarios.logging.logger_constants as log_const
from scenarios.actions.action import ClearCurrentScenarioAction
from scenarios.user.user_model import User

from smart_kit.handlers.handler_base import HandlerBase


class HandlerCloseApp(HandlerBase):
    def __init__(self, app_name: str):
        super().__init__(app_name)
        self._clear_current_scenario = ClearCurrentScenarioAction()

    def run(self, payload: Dict[str, Any], user: User) -> List[Command]:
        commands = super().run(payload, user)
        text_preprocessing_result = TextPreprocessingResult.from_payload(payload)
        params = {
            log_const.KEY_NAME: "HandlerCloseApp"
        }
        self._clear_current_scenario.run(user, text_preprocessing_result)
        if payload.get("message"):
            params["tpr_str"] = str(text_preprocessing_result.raw)
        log("HandlerCloseApp with text preprocessing result", user, params)
        return commands



# coding: utf-
from typing import Dict, Any, List

import scenarios.logging.logger_constants as log_const
from core.basic_models.actions.command import Command
from core.logging.logger_utils import log
from core.text_preprocessing.preprocessing_result import TextPreprocessingResult
from scenarios.user.user_model import User

from smart_kit.handlers.handler_base import HandlerBase
from smart_kit.models.dialogue_manager import DialogueManager


class HandlerRunApp(HandlerBase):
    def __init__(self, app_name: str, dialogue_manager: DialogueManager):
        super().__init__(app_name)
        log(
            f"{self.__class__.__name__}.__init__ started.", params={log_const.KEY_NAME: log_const.STARTUP_VALUE}
        )
        self.dialogue_manager = dialogue_manager
        log(
            f"{self.__class__.__name__}.__init__ finished.", params={log_const.KEY_NAME: log_const.STARTUP_VALUE}
        )

    def run(self, payload: Dict[str, Any], user: User) -> List[Command]:
        commands = super().run(payload, user)

        params = {log_const.KEY_NAME: "handling_run_app"}
        log(f"{self.__class__.__name__}.run started", user, params)

        commands.extend(self._handle_base(user))
        return commands

    def _handle_base(self, user: User) -> List[Command]:
        answer, is_answer_found = self.dialogue_manager.run(TextPreprocessingResult({}), user)
        return answer or []



# coding=utf-8
from typing import Callable


class DescriptionsItems:
    def __init__(self, factory: Callable, items, ordered=False):
        items = items or {}
        self._factory = factory
        self._raw_items = None
        self._items = None
        self._init(items)

    def __contains__(self, key):
        return key in self._raw_items

    def _init(self, raw_items):
        self._raw_items = raw_items
        self._items = dict()
        for id in self._raw_items.keys():
            self._get_or_create_item(id)

    def __getitem__(self, id):
        return self._get_or_create_item(id)

    def _get_or_create_item(self, id):
        existed_item = self._items.get(id)
        if existed_item is None:
            existed_item = self._factory(id=id, items=self._raw_items[id])
            self._items[id] = existed_item
        return existed_item

    # should implement python dictionary interface
    def get(self, key):
        if key in self:
            return self.__getitem__(key)

    def __len__(self):
        return len(self._raw_items)

    def __iter__(self):
        return iter(self._raw_items)

    def get_keys(self):
        return self._raw_items.keys()

    def keys(self):
        return self._raw_items.keys()

    def update_data(self, items):
        self._init(items)

    def update_item(self, key, item):
        if key in self._items:
            del self._items[key]
        self._raw_items[key] = item
        self._get_or_create_item(key)

    def remove_item(self, key):
        if key in self._items:
            del self._items[key]
        if key in self._raw_items:
            del self._raw_items[key]



import redis
import typing
from core.db_adapter.db_adapter import DBAdapter
from core.db_adapter import error


class RedisAdapter(DBAdapter):
    def __init__(self, config=None):
        super().__init__(config)
        self._redis: typing.Optional[redis.Redis] = None

        try:
            del self.config["type"]
        except KeyError:
            pass

    def connect(self):
        self._redis = redis.Redis(**self.config)

    def _open(self, filename, *args, **kwargs):
        pass

    def _save(self, id, data):
        return self._redis.set(id, data)

    def _replace_if_equals(self, id, sample, data):
        return self._redis.set(id, data)

    def _get(self, id):
        data = self._redis.get(id)
        return data.decode() if data else None

    def _list_dir(self, path):
        raise error.NotSupportedOperation

    def _glob(self, path, pattern):
        raise error.NotSupportedOperation

    def _path_exists(self, path):
        self._redis.exists(path)

    def _on_prepare(self):
        pass



from typing import Dict, Any, Optional, Union, List

from core.basic_models.actions.command import Command
from core.basic_models.actions.string_actions import StringAction
from core.text_preprocessing.base import BaseTextPreprocessingResult
from core.utils.pickle_copy import pickle_deepcopy
from scenarios.user.user_model import User
from smart_kit.names.message_names import GET_PROFILE_DATA


class SmartGeoAction(StringAction):
    """Action для получения геоданных пользователя

    Описание smart geo сервиса: https://developers.sber.ru/docs/ru/va/reference/smartservices/smartgeo/overview
    Условия подключение smart geo: https://developers.sber.ru/docs/ru/va/reference/smartservices/smartgeo/conditions
    Описание smart geo API: https://developers.sber.ru/docs/ru/va/reference/smartservices/smartgeo/api

    ## Использование:
    Внимание: action поддерживает только смартаппы, использующие `MAIN_LOOP = MainLoop` (задаётся в _app_config.py_),
    где `from smart_kit.start_points.main_loop_kafka import MainLoop`.
    ```
    {
      "type": "smart_geo",
      "behavior": "smart_geo_behavior"
    }
    ```
    Опциональный параметр `"behavior"` определяет Behavior, используемый в запросе.

    После исполнения команды и успешного ответа от smart geo API геоданные пользователя сохранятся в переменную внутри
    User `user.variables.smart_geo`
    """
    def __init__(self, items: Dict[str, Any], id: Optional[str] = None):
        super().__init__(items, id)
        self.command = GET_PROFILE_DATA
        self.behavior = items.get("behavior", "smart_geo_behavior")

    def run(self, user: User, text_preprocessing_result: BaseTextPreprocessingResult,
            params: Optional[Dict[str, Union[str, float, int]]] = None) -> List[Command]:
        scenario_id = user.last_scenarios.last_scenario_name
        user.behaviors.add(user.message.generate_new_callback_id(), self.behavior, scenario_id,
                           text_preprocessing_result.raw, action_params=pickle_deepcopy(params))

        commands = super().run(user, text_preprocessing_result, params)
        return commands



import unittest

from scenarios.user.preprocessing_messages.prepricessing_messages_for_scenarios import \
    PreprocessingScenariosMessages
from smart_kit.utils.picklable_mock import PicklableMock


class PreprocessingScenariosMessagesTest(unittest.TestCase):

    def test_add_1(self):
        user = PicklableMock()
        items = None
        description = PicklableMock()
        description.max_message_count = 3
        description.lifetime = 10
        preprocessing_messages = PreprocessingScenariosMessages(items, description, user)
        text_preprocessing_result = PicklableMock()
        text_preprocessing_result.raw = {"test": 123}
        preprocessing_messages.add(text_preprocessing_result)
        self.assertEqual([item for item in preprocessing_messages.processed_items], [{"test": 123}])

    def test_add_2(self):
        user = PicklableMock()
        items = [{"ts": 35343820800, "message": {"test": 567}, "direction": "incoming"}]
        description = PicklableMock()
        description.max_message_count = 3
        description.lifetime = 10
        preprocessing_messages = PreprocessingScenariosMessages(items, description, user)
        text_preprocessing_result = PicklableMock()
        text_preprocessing_result.raw = {"test": 123}
        preprocessing_messages.add(text_preprocessing_result)
        self.assertListEqual([item for item in preprocessing_messages.processed_items], [{"test": 123}, {"test": 567}])

    def test_add_3(self):
        user = PicklableMock()
        items = [{"ts": 35343820800, "message": {"test": 567}, "direction": "incoming"}]
        description = PicklableMock()
        description.max_message_count = 1
        description.lifetime = 10
        preprocessing_messages = PreprocessingScenariosMessages(items, description, user)
        text_preprocessing_result = PicklableMock()
        text_preprocessing_result.raw = {"test": 123}
        preprocessing_messages.add(text_preprocessing_result)
        self.assertListEqual([item for item in preprocessing_messages.processed_items], [{"test": 123}])

    def test_clear(self):
        user = PicklableMock()
        items = [{"ts": 35343820800, "message": {"test": 567}, "direction": "incoming"}]
        description = PicklableMock()
        description.max_message_count = 1
        description.lifetime = 10
        preprocessing_messages = PreprocessingScenariosMessages(items, description, user)
        preprocessing_messages.clear()
        self.assertListEqual([item for item in preprocessing_messages.processed_items], [])



KEY_NAME = "key_name"
SCENARIO_VALUE = "scenario"
REQUIREMENT_CHECK_VALUE = "requirement_check"
REQUIREMENT_TRACE_VALUE = "requirement_trace"
TEMPLATE_TRACE_VALUE = "jinja2_trace"
EXCEPTION_VALUE = "exception"
HANDLED_EXCEPTION_VALUE = "handled_exception"
IGNITE_VALUE = "ignite"
MEMCACHED_VALUE = "memcached"
TWISTED_SERVER = "twisted"
REPOSITORY_LOAD_VALUE = "repository_load"
REPOSITORY_CLEAR_VALUE = "repository_clear"
KAFKA_ON_ASSIGN_VALUE = "kafka_on_assign"
IR_STARTUP_VALUE = "intent_recognizer_startup"
INTENT_ID_VALUE = "intent_id"
REQUEST_VALUE = "request"



# coding: utf-8
import time
import unittest
from unittest.mock import Mock

from scenarios.behaviors import behaviors
from smart_kit.utils.picklable_mock import PicklableMock


class BehaviorsTest3(unittest.TestCase):
    def setUp(self):
        self.test_items1 = {"123-ASDF": {"behavior_id": 11, "expire_time": time.time() + 10, "scenario_id": 22,
                                         "text_preprocessing_result": {}, "action_params": {}}}
        self.test_items2 = {"123-ASDF": {"behavior_id": 11, "expire_time": time.time(), "scenario_id": 22,
                                         "text_preprocessing_result": {}, "action_params": {}}}
        self.test_user = Mock('User')
        self.test_user.id = 111
        self.test_user.message = PicklableMock()
        self.test_user.message.incremental_id = 123
        self.test_user.message.logging_uuid = 222
        self.test_user.settings = PicklableMock()
        self.test_user.settings.app_name = "test"
        self.test_descriptions = Mock('Descriptions')

    def test_behaviors_init(self):
        obj = behaviors.Behaviors(self.test_items1, self.test_descriptions, self.test_user)
        obj.initialize()
        self.assertTrue(hasattr(obj, 'timeout'))
        self.assertTrue(hasattr(obj, '_delete'))
        self.assertTrue(list(obj._callbacks.keys()) == ['123-ASDF'])

    def test_behaviors_expire(self):
        obj1 = behaviors.Behaviors(self.test_items1, self.test_descriptions, self.test_user)
        obj2 = behaviors.Behaviors(self.test_items2, self.test_descriptions, self.test_user)
        source_value1 = list(obj1._callbacks.keys())
        # source_value2 = list(obj1._callbacks.keys())
        obj1.expire()
        obj2.expire()
        self.assertTrue(list(obj1._callbacks.keys()) == source_value1)
        self.assertTrue(list(obj2._callbacks.keys()) == [])

    def test_behaviors_has_callback(self):
        obj1 = behaviors.Behaviors(self.test_items1, self.test_descriptions, self.test_user)
        obj1.initialize()
        self.assertTrue(obj1.has_callback('123-ASDF'))

    def test_behavoirs_log_const(self):
        self.assertIsNotNone(behaviors.log_const.KEY_NAME)
        self.assertIsNotNone(behaviors.log_const.BEHAVIOR_CALLBACK_ID_VALUE)
        self.assertIsNotNone(behaviors.log_const.BEHAVIOR_DATA_VALUE)



from typing import Dict, Any, Optional, Union, List

from core.basic_models.actions.command import Command
from core.basic_models.actions.string_actions import StringAction
from core.configs.global_constants import KAFKA
from core.text_preprocessing.base import BaseTextPreprocessingResult
from core.utils.pickle_copy import pickle_deepcopy
from scenarios.user.user_model import User
from smart_kit.configs import settings

GIVE_ME_MEMORY = "GIVE_ME_MEMORY"
REMEMBER_THIS = "REMEMBER_THIS"


class GiveMeMemoryAction(StringAction):
    """
    Example::
        {
            "type": "give_me_memory",
            "behavior": "client_info_request",
            "nodes": {
                "memory": {
                    "confidentialMemo": [
                        "userAgreement"
                    ],
                    "projectPrivateMemo": [
                        "character_{{message.uuid.userId}}"
                    ]
                },
                "profileEmployee": {
                    "type": "unified_template",
                    "template": "{{ 0 }}",
                    "loader": "json"
                },
                "tokenType": {
                    "type": "unified_template",
                    "template": "{{ 0 }}",
                    "loader": "json"
                }
            }
        }
    """
    DEFAULT_KAFKA_KEY = "main"

    def __init__(self, items: Dict[str, Any], id: Optional[str] = None):
        super().__init__(items, id)
        config = settings.Settings()
        self.command = GIVE_ME_MEMORY
        self.request_type = KAFKA
        self.kafka_key: Optional[str] = items.get("kafka_key")
        self.behavior: Optional[str] = items.get("behavior")
        self._nodes.update({
            "root_nodes": {
                "protocolVersion": items.get("protocolVersion") or 1
            },
            "memory": [
                {"memoryPartition": key, "tags": val} for key, val in self._nodes["memory"].items()
            ],
            "consumer": {
                "projectId": config["template_settings"]["project_id"]
            }
        })
        settings_kafka_key = config["template_settings"].get("client_profile_kafka_key")
        self.kafka_key: str = self.kafka_key or settings_kafka_key or self.DEFAULT_KAFKA_KEY
        self.request_data = {
            "topic_key": "client_info",
            "kafka_key": self.kafka_key,
            "kafka_replyTopic":
                config["template_settings"]["consumer_topic"]
        }

    def run(self, user: User, text_preprocessing_result: BaseTextPreprocessingResult,
            params: Optional[Dict[str, Union[str, float, int]]] = None) -> [List[Command]]:
        if self.behavior:
            callback_id = user.message.generate_new_callback_id()
            scenario_id = user.last_scenarios.last_scenario_name if hasattr(user, 'last_scenarios') else None
            user.behaviors.add(callback_id, self.behavior, scenario_id,
                               text_preprocessing_result.raw, pickle_deepcopy(params))

        commands = super().run(user, text_preprocessing_result, params)
        return commands


class RememberThisAction(StringAction):
    """
    Example::
      {
        "type": "remember_this",
        "nodes": {
          "clientIds": {
            "type": "unified_template",
            "template": "{{ 0 }}",
            "loader": "json"
          },
          "memory": [
            {
              "memoryPartition": "publicMemo",
              "partitionData": [
                {
                  "tag": "historyInfo",
                  "action": {
                    "type": "upsert",
                    "params": {
                      "operation": [
                        {
                          "selector": {
                            "intent": {
                              "$eq": "run_app"
                            },
                            "surface": {
                              "$eq": "{{ 0 }}"
                            },
                            "channel": {
                              "$eq": "{{ 0 }}"
                            },
                            "projectId": {
                              "$eq": "{{ 0 }}"
                            }
                          },
                          "updater": [
                            {
                              "$set": {
                                "$.lastExecuteDateTime": "{{ 0 }}"
                              }
                            },
                            {
                              "$inc": {
                                "$.executeCounter": 1
                              }
                            }
                          ]
                        }
                      ]
                    }
                  }
                }
              ]
            }
          ]
        }
      }
    """
    DEFAULT_KAFKA_KEY = "main"

    def __init__(self, items: Dict[str, Any], id: Optional[str] = None):
        super().__init__(items, id)
        self.command = REMEMBER_THIS
        self.request_type = KAFKA
        self.kafka_key: Optional[str] = items.get("kafka_key")
        self._nodes.update({
            "root_nodes": {
                "protocolVersion": items.get("protocolVersion", 3)
            }
        })

    def run(self, user: User, text_preprocessing_result: BaseTextPreprocessingResult,
            params: Optional[Dict[str, Union[str, float, int]]] = None) -> Optional[List[Command]]:
        self._nodes.update({
            "consumer": {
                "projectId": user.settings["template_settings"]["project_id"]
            }
        })
        settings_kafka_key: Optional[str] = user.settings["template_settings"].get("client_profile_kafka_key")
        kafka_key: str = self.kafka_key or settings_kafka_key or self.DEFAULT_KAFKA_KEY
        self.request_data = {
            "topic_key": "client_info_remember",
            "kafka_key": kafka_key,
            "kafka_replyTopic":
                user.settings["template_settings"]["consumer_topic"]
        }

        commands = super().run(user, text_preprocessing_result, params)
        return commands



# coding: utf-8
import unittest

from smart_kit.message import app_info


class MessageAppInfoTest1(unittest.TestCase):
    def test_app_info1(self):
        obj = app_info.AppInfo(dict())
        self.assertIsNone(obj.project_id)
        self.assertIsNone(obj.application_id)
        self.assertIsNone(obj.app_version_id)
        self.assertIsNone(obj.frontend_endpoint)
        self.assertIsNone(obj.frontend_type)

    def test_app_info2(self):
        obj = app_info.AppInfo({"projectId": 1, "applicationId": 1, "appversionId": 1, "frontendEndpoint": 1,
                                "frontendType": 1})
        self.assertTrue(obj.project_id == 1)
        self.assertTrue(obj.application_id == 1)
        self.assertTrue(obj.app_version_id == 1)
        self.assertTrue(obj.frontend_endpoint == 1)
        self.assertTrue(obj.frontend_type == 1)



# coding: utf-8
import time
from unittest import TestCase

from scenarios.scenario_models.field.field import field_models, QuestionField
from scenarios.scenario_models.forms.form import form_models, Form
from scenarios.scenario_models.forms.forms import Forms
from smart_kit.utils.picklable_mock import PicklableMock


class MockDescription:
    def __init__(self, id, check_expired, fields=None):
        fields = fields or {}
        self.id = id
        self.check_expired = check_expired
        self.lifetime = 5
        self.valid = True
        self.fields = fields
        self.keep_forms_alive = False
        self.form_type = id


class MockDescriptions:
    def __init__(self, items, *args, **kwargs):
        self.items = items

    def __getitem__(self, key):
        return self.items[key]

    def get(self, key):
        return self.items.get(key)

    def __contains__(self, key):
        return key in self.items


class MockField:
    def __init__(self, id):
        self.id = id
        self.available = True
        self.need_load_context = False
        self.default_value = None


class MockForm:
    def __init__(self, value, description, user):
        self.value = value
        self.description = description

    def check_expired(self):
        return self.description.check_expired

    @property
    def raw(self):
        return self.value


class FormsTest(TestCase):

    def setUp(self):
        self.mock_0 = {}
        self.mock_1 = {'sbm_credit': {'fields': {'amount': {'value': 100.0},
                                                 'currency': {"required": True,
                                                              "filler": {"type": "currency_first"},
                                                              "questions": [{"type": "external",
                                                                             "action": "sbm_currency_question"}]
                                                              }},
                                      'remove_time': 1506418333},
                       'Turn_on_MB': {'remove_time': 2506421370}}

    def test_raw(self):
        user = PicklableMock()
        forms = Forms(self.mock_1, {"sbm_credit": PicklableMock(), "Turn_on_MB": PicklableMock()}, user)
        expected_dict = {'sbm_credit': {'fields': {'amount': {'value': 100.0},
                                                   'currency': {"required": True,
                                                                "filler": {"type": "currency_first"},
                                                                "questions": [{"type": "external",
                                                                               "action": "sbm_currency_question"}]
                                                                }},
                                        'remove_time': 1506418333},
                         'Turn_on_MB': {'remove_time': 2506421370}}
        self.assertDictEqual(forms.raw, expected_dict)

    def test_new(self):
        user = PicklableMock()
        form_models[MockDescription] = Form
        description_MB = MockDescription('Turn_on_MB', False)
        description_credit = MockDescription('sbm_credit', False)
        description_off_MB = MockDescription('Turn_off_MB', False)
        descriptions = {'Turn_on_MB': description_MB, 'sbm_credit': description_credit,
                        'Turn_off_MB': description_off_MB}
        forms = Forms(self.mock_1, descriptions, user)
        form = forms.new('Turn_off_MB')
        ts = int(time.time()) + form.description.lifetime
        expected_dict = {'sbm_credit': {'fields': {'amount': {'value': 100.0},
                                                   'currency': {"required": True,
                                                                "filler": {"type": "currency_first"},
                                                                "questions": [{"type": "external",
                                                                               "action": "sbm_currency_question"}]
                                                                }},
                                        'remove_time': 1506418333},
                         'Turn_on_MB': {'remove_time': 2506421370},
                         'Turn_off_MB': {'remove_time': ts}}
        self.assertDictEqual(forms.raw, expected_dict)

    def test_remove(self):
        user = PicklableMock()
        forms = Forms(self.mock_1, {"sbm_credit": PicklableMock(), "Turn_on_MB": PicklableMock()}, user)
        forms.remove_item("sbm_credit")
        expected_dict = {"Turn_on_MB": {'remove_time': 2506421370}}
        self.assertDictEqual(forms.raw, expected_dict)

    def test_expire(self):
        user = PicklableMock()
        form_models[MockDescription] = MockForm
        description_MB = MockDescription("Turn_on_MB", False)
        description_credit = MockDescription("sbm_credit", True)
        descriptions = {"Turn_on_MB": description_MB, "sbm_credit": description_credit}
        forms = Forms(self.mock_1, descriptions, user)
        forms.expire()
        expected_dict = {"Turn_on_MB": {'remove_time': 2506421370}}
        self.assertDictEqual(forms.raw, expected_dict)

    def test_expire0(self):
        user = PicklableMock()
        form = PicklableMock()
        form.check_expired = False
        form_models[MockDescription] = MockForm
        description_MB = MockDescription("Turn_on_MB", True)
        description_credit = MockDescription("sbm_credit", True)
        descriptions = {"Turn_on_MB": description_MB, "sbm_credit": description_credit}
        forms = Forms(self.mock_1, descriptions, user)
        forms.expire()
        expected_dict = {}
        self.assertDictEqual(forms.raw, expected_dict)

    def test_clear(self):
        user = PicklableMock()
        forms = Forms(self.mock_1, {"mock_descr": PicklableMock()}, user)
        forms.clear_all()
        expected_dict = {}
        self.assertDictEqual(forms.raw, expected_dict)

    def test_collect_form_fields(self):
        user = PicklableMock()
        user.settings = {"template_settings": {}}
        form_models[MockDescription] = Form
        field_models[MockField] = QuestionField
        field1 = MockField("amount")
        field2 = MockField("currency")
        fields_descriptions = {"amount": field1, "currency": field2}
        description_MB = MockDescription('Turn_on_MB', False, fields_descriptions)
        description_credit = MockDescription('sbm_credit', False, fields_descriptions)
        description_off_MB = MockDescription('Turn_off_MB', False, fields_descriptions)
        _descriptions = {'Turn_on_MB': description_MB, 'sbm_credit': description_credit,
                        'Turn_off_MB': description_off_MB}
        descriptions = MockDescriptions(_descriptions)
        forms = Forms(self.mock_1, descriptions, user)
        result = forms.collect_form_fields()
        expected_fields = set(self.mock_1.keys())
        self.assertSetEqual(set(result.keys()), expected_fields)

    def test_clear_form(self):
        expected_result = {'sbm_credit': {'fields': {'amount': {'value': 100.0},
                                                     'currency': {'filler': {'type': 'currency_first'},
                                                                  'questions': [ {'action': 'sbm_currency_question',
                                                                                  'type': 'external'} ],
                                                                  'required': True}},
                                          'remove_time': 1506418333}}
        description_MB = MockDescription('Turn_on_MB', False)
        description_credit = MockDescription('sbm_credit', False)
        description_off_MB = MockDescription('Turn_off_MB', False)
        descriptions = {"scenarios": MockDescriptions({'Turn_on_MB': description_MB,
                                                       'sbm_credit': description_credit,
                                                       'Turn_off_MB': description_off_MB}),
                        "forms": MockDescriptions({'Turn_on_MB': description_MB,
                                                       'sbm_credit': description_credit,
                                                       'Turn_off_MB': description_off_MB})
                        }
        user = PicklableMock()
        user.descriptions = descriptions
        forms = Forms(self.mock_1, descriptions["forms"], user)
        user.forms = forms
        forms.clear_form("Turn_on_MB")
        self.assertEqual(expected_result, user.forms.raw)



# coding: utf-8
import unittest
from unittest.mock import Mock

from core.model.field import Field
from scenarios.user import user_model
from smart_kit.utils.picklable_mock import PicklableMock


class UserTest2(unittest.TestCase):
    def setUp(self):
        self.test_scenario1 = Mock('scenario')
        self.test_scenario1.scenario_description = "This is test scenario 1 desc"
        self.test_scenario1.form_type = "any type 1"
        self.test_id = 11
        self.test_message = PicklableMock()
        self.test_message.incremental_id = 123
        self.test_values = {11: "any val"}
        history = PicklableMock()
        history.enabled = True
        self.test_descriptions = {"forms": {}, "last_scenarios": [11], "scenarios": [self.test_scenario1],
                                  "preprocessing_messages_for_scenario": "any msg",
                                  "preprocessing_messages_for_scenarios": "any msg", "behaviors": [],
                                  "last_action_ids": [], "history": history, "bundles": {}}
        self.test_parametrizer_cls = lambda x, y: y

    def test_smart_app_user_init(self):
        obj1 = user_model.User(self.test_id, self.test_message, None, self.test_values, self.test_descriptions,
                                       self.test_parametrizer_cls)
        obj2 = user_model.User(self.test_id, self.test_message, None, self.test_values, self.test_descriptions,
                                       self.test_parametrizer_cls, True)
        self.assertTrue(len(obj1.fields) > 0)
        self.assertTrue(obj1.id == self.test_id)
        self.assertTrue(not obj1.do_not_save)
        self.assertTrue(obj2.load_error)

    def test_smart_app_user_fields(self):
        obj1 = user_model.User(self.test_id, self.test_message, None, self.test_values, self.test_descriptions,
                                       self.test_parametrizer_cls)
        self.assertTrue(len(obj1.fields) == 14)
        self.assertTrue(isinstance(obj1.fields[0], Field))

    def test_smart_app_user_parametrizer(self):
        obj1 = user_model.User(self.test_id, self.test_message, None, self.test_values, self.test_descriptions,
                                       self.test_parametrizer_cls)
        self.assertTrue(obj1.parametrizer == {})



# coding: utf-8
import time
from collections import deque


class MessageHistory:
    TS = "ts"
    MESSAGE = "message"
    DIRECTION = "direction"
    INCOMING = "incoming"
    OUTGOING = "outgoing"

    def __init__(self, items, description, user):
        items = items or []
        self.items = deque(items)
        self.description = description

    def _push(self, message, direction):
        self._filter()
        stored_data = {self.TS: time.time(),
                       self.MESSAGE: message,
                       self.DIRECTION: direction}
        self.items.append(stored_data)

    def clear(self):
        self.items = deque()

    def _filter(self):
        now = time.time()
        while len(self.items) >= self.description.max_message_count:
            self.items.popleft()

        for _ in range(len(self.items)):
            threshold = self.description.lifetime + self.items[0].get(self.TS)
            if now > threshold:
                self.items.popleft()
            else:
                break

    @property
    def raw(self):
        return list(self.items)



from functools import cached_property
from typing import Iterable
import json
from copy import copy

from core.utils.masking_message import masking
from core.message.msg_validator import MessageValidator
from smart_kit.utils import SmartAppToMessage_pb2


class SmartAppToMessage:
    ROOT_NODES_KEY = "root_nodes"
    PAYLOAD = "payload"

    def __init__(self, command, message, request, forward_fields=None, masking_fields=None,
                 validators: Iterable[MessageValidator] = ()):
        root_nodes = command.payload.pop(self.ROOT_NODES_KEY, None)
        self.command = command
        self.root_nodes = root_nodes or {}
        self.incoming_message = message
        self.request = request
        self.forward_fields = forward_fields or ()
        self.masking_fields = masking_fields
        self.validators = validators

    @cached_property
    def payload(self):
        payload = copy(self.command.payload)
        for field in self.forward_fields:
            if field not in self.incoming_message.payload:
                continue
            if field in payload:
                continue
            payload[field] = self.incoming_message.payload[field]
        return payload

    @cached_property
    def as_dict(self):
        fields = {
            "messageId": self.incoming_message.incremental_id,
            "sessionId": self.incoming_message.session_id,
            "messageName": self.command.name,
            "payload": self.payload,
            "uuid": self.incoming_message.uuid
        }
        fields.update(self.root_nodes)
        return fields

    @staticmethod
    def as_protobuf_message(data_as_dict):
        message = SmartAppToMessage_pb2.SmartAppToMessage()
        message.messageId = data_as_dict["messageId"]
        message.sessionId = data_as_dict["sessionId"]
        message.messageName = data_as_dict["messageName"]
        message.payload = data_as_dict["payload"]
        uuid = message.uuid.add()
        uuid.userId = data_as_dict["uuid"]["userId"]
        uuid.userChannel = data_as_dict["uuid"]["userChannel"]
        return message

    @cached_property
    def masked_value(self):
        masked_data = masking(self.as_dict, self.masking_fields)
        if self.command.loader == "json.dumps":
            return json.dumps(masked_data, ensure_ascii=False)
        elif self.command.loader == "protobuf":
            protobuf_message = self.as_protobuf_message(masked_data)
            return protobuf_message.SerializeToString()

    @cached_property
    def value(self):
        if self.command.loader == "json.dumps":
            return json.dumps(self.as_dict, ensure_ascii=False)
        elif self.command.loader == "protobuf":
            protobuf_message = self.as_protobuf_message(self.as_dict)
            return protobuf_message.SerializeToString()

    def validate(self):
        for validator in self.validators:
            if not validator.validate(self.command.name, self.payload):
                return False
        return True



# coding: utf-8
import timeit


class StatsTimer:
    def __enter__(self):
        self.start = timeit.default_timer()
        return self

    def __exit__(self, *args):
        self.end = timeit.default_timer()
        self.secs = self.end - self.start
        self.msecs = self.secs * 1000



# coding: utf-8
from core.descriptions.descriptions_items import DescriptionsItems
from scenarios.user.last_scenarios.last_scenarios_description import LastScenariosDescription


class LastScenariosDescriptionsItems(DescriptionsItems):
    DEFAULT_COUNT = 1

    def __init__(self, items):
        super(LastScenariosDescriptionsItems, self).__init__(LastScenariosDescription, items)

    def get_count(self, text_preprocessing_result, user):
        count = self.DEFAULT_COUNT
        for key in self:
            last_scenario_description = self[key]
            if last_scenario_description.check(text_preprocessing_result, user):
                count = last_scenario_description.count
        return count



# coding: utf-8
import unittest
from time import time

from core.basic_models.counter.counter import Counter
from core.basic_models.counter.counters import Counters


class CounterTest(unittest.TestCase):

    def _create_counter(self, value, lifetime=None, time_shift=0):
        current_time = int(time()) + time_shift
        self.items = {
            "value": value,
            "create_time": current_time,
            "update_time": current_time,
            "lifetime": lifetime
        }
        return Counter(self.items)

    def test_empty_items(self):
        counter = Counter({})
        self.assertIsNone(counter.value)
        self.assertLessEqual(counter.create_time, int(time()))
        self.assertIsNone(counter.update_time)
        self.assertIsNone(counter.lifetime)

    def test_check_expire_true(self):
        lifetime = 100
        time_shift = - (lifetime + 1)
        counter = self._create_counter(2, lifetime, time_shift)
        self.assertTrue(counter.check_expire())

    def test_check_expire_false(self):
        lifetime = 1000
        counter = self._create_counter(2, lifetime)
        self.assertFalse(counter.check_expire())

    def test_check_expire_none(self):
        counter = Counter({})
        self.assertFalse(counter.check_expire())

    def test_inc_positive(self):
        value = 2
        increment = 3
        counter = self._create_counter(value)
        counter.inc(increment)
        self.assertEqual(counter.value, (value + increment))

    def test_inc_negative(self):
        value = 3
        increment = -2
        counter = self._create_counter(value, time_shift=-10)
        counter.inc(increment)
        self.assertEqual(counter.value, value + increment)
        self.assertLess(counter.create_time, counter.update_time)

    def test_inc_from_empty(self):
        increment = 3
        items = {}
        counter = Counter(items)
        counter.inc(increment)
        self.assertEqual(counter.value, increment)
        self.assertLessEqual(counter.create_time, time())
        self.assertLessEqual(counter.update_time, time())
        self.assertIsNone(counter.lifetime)

    def test_dec(self):
        value = 3
        counter = self._create_counter(value)
        counter.dec()
        self.assertEqual(counter.value, value - 1)

    def test_dec_from_empty(self):
        items = {}
        counter = Counter(items)
        counter.dec()
        self.assertEqual(counter.value, -1)
        self.assertLessEqual(counter.create_time, time())
        self.assertLessEqual(counter.update_time, time())
        self.assertIsNone(counter.lifetime)

    def test_inc_not_create_lifetime(self):
        lifetime = 100
        counter = self._create_counter(value=0, lifetime=lifetime)
        counter.inc(lifetime=200)
        self.assertEqual(counter.lifetime, lifetime)

    def test_inc_create_lifetime(self):
        lifetime = 100
        counter = Counter({})
        counter.inc(lifetime=lifetime)
        self.assertEqual(counter.lifetime, lifetime)

    def test_set(self):
        value = 3
        new_value = 7
        counter = self._create_counter(value, time_shift=-10)
        counter.set(new_value)
        self.assertEqual(counter.value, new_value)
        self.assertLess(counter.create_time, counter.update_time)
        self.assertLessEqual(counter.update_time, time())

    def test_set_time_shift(self):
        value = 3
        new_value = 7
        time_shift = 100
        counter = self._create_counter(value)
        counter.set(new_value, time_shift=time_shift)
        self.assertEqual(counter.value, new_value)
        self.assertGreaterEqual(counter.update_time - int(time()), time_shift)

    def test_set_reset_time(self):
        value = 3
        new_value = 7
        time_shift = 100
        counter = self._create_counter(value)
        counter.set(new_value, reset_time=True)
        self.assertEqual(counter.value, new_value, time_shift)
        self.assertEqual(counter.create_time, counter.update_time)

    def test_raw(self):
        counter = self._create_counter(2)
        self.assertDictEqual(counter.raw, self.items)

    def test_eq(self):
        value = 3
        counter = self._create_counter(value)
        self.assertTrue(counter == value)

    def test_not_eq_empty_counter(self):
        value = 3
        counter = Counter({})
        self.assertFalse(counter == value)

    def test_ne(self):
        value = 3
        counter = self._create_counter(value)
        self.assertTrue(counter != 4)

    def test_lt(self):
        value = 3
        counter = self._create_counter(value)
        self.assertTrue(counter < 4)

    def test_lt_empty(self):
        counter = Counter({})
        self.assertTrue(counter < 4)

    def test_gt(self):
        value = 4
        counter = self._create_counter(value)
        self.assertTrue(counter > 3)

    def test_ge(self):
        value = 4
        counter = self._create_counter(value)
        self.assertTrue(counter >= 3)

    def test_le(self):
        value = 3
        counter = self._create_counter(value)
        self.assertTrue(counter <= 4)


class CountersTest(unittest.TestCase):

    def test_get_not_exist(self):
        name = "test_counter"
        counters = Counters({}, None)
        counter = counters.get(name)
        self.assertIsNone(counter.raw)
        self.assertEqual(counter.value, None)
        self.assertEqual(counter.create_time, int(time()))
        self.assertEqual(counter.update_time, None)
        self.assertEqual(counter.lifetime, None)

    def test_get_from_raw_items(self):
        current_time = int(time())
        name = "test_counter"
        raw_data = {name: {
            "value": 5,
            "create_time": current_time,
            "update_time": current_time
        }}
        counters = Counters(raw_data, None)
        counter = counters.get(name)
        self.assertEqual(counter.value, 5)
        self.assertEqual(counter.create_time, current_time)
        self.assertEqual(counter.update_time, current_time)
        self.assertEqual(counters.raw.get(name)["value"], 5)

    def test_get_exist(self):
        current_time = int(time())
        name = "test_counter"
        counter_data = {
            "value": 5,
            "create_time": current_time,
            "update_time": current_time
        }
        counters = Counters({}, None)
        test_counter = Counter(counter_data)
        counters._items[name] = test_counter
        counter = counters.get(name)
        self.assertEqual(counter.value, 5)
        self.assertEqual(counters.raw.get(name)["value"], 5)

    def test_raw(self):
        time_shift = 100
        current_time = int(time())
        counter_1 = {
            "value": 1,
            "create_time": current_time,
            "update_time": current_time,
            "lifetime": None
        }
        current_time = current_time + time_shift
        counter_2 = {
            "value": 2,
            "create_time": current_time,
            "update_time": current_time,
            "lifetime": None
        }
        name_1 = "test_counter_1"
        name_2 = "test_counter_2"
        counters = Counters({}, None)
        test_counter_1 = Counter(counter_1)
        test_counter_2 = Counter(counter_2)
        counters._items[name_1] = test_counter_1
        counters._items[name_2] = test_counter_2
        self.assertDictEqual(counters.raw, { name_1: counter_1, name_2: counter_2})

    def test_clear(self):
        time_shift = 100
        current_time = int(time())
        counter_1 = {
            "value": 1,
            "create_time": current_time,
            "update_time": current_time,
            "lifetime": None
        }
        current_time = current_time + time_shift
        counter_2 = {
            "value": 2,
            "create_time": current_time,
            "update_time": current_time,
            "lifetime": None
        }
        name_1 = "test_counter_1"
        name_2 = "test_counter_2"
        counters = Counters({name_1: counter_1, name_2: counter_2}, None)
        counters.clear(name_2)
        self.assertDictEqual(counters.raw, {name_1: counter_1})

    def test_clear_not_exist(self):
        current_time = int(time())
        counter_1 = {
            "value": 1,
            "create_time": current_time,
            "update_time": current_time,
            "lifetime": None
        }
        name_1 = "test_counter_1"
        name_2 = "test_counter_2"
        counters = Counters({name_1: counter_1}, None)
        counters.clear(name_2)
        self.assertDictEqual(counters.raw, {name_1: counter_1})

    def test_expire_some(self):
        lifetime = 100
        current_time = int(time())
        counter_1 = {
            "value": 1,
            "create_time": current_time,
            "update_time": current_time,
            "lifetime": lifetime
        }
        current_time = current_time - lifetime - 1
        counter_2 = {
            "value": 2,
            "create_time": current_time,
            "update_time": current_time,
            "lifetime": lifetime
        }
        name_1 = "test_counter_1"
        name_2 = "test_counter_2"
        counters = Counters({name_1: counter_1, name_2: counter_2}, None)
        counters.expire()
        self.assertDictEqual(counters.raw, {name_1: counter_1})

    def test_expire_all(self):
        lifetime = 100
        current_time = int(time()) - lifetime - 1
        counter_1 = {
            "value": 1,
            "create_time": current_time,
            "update_time": current_time,
            "lifetime": lifetime
        }

        counter_2 = {
            "value": 2,
            "create_time": current_time,
            "update_time": current_time,
            "lifetime": lifetime - 20
        }
        name_1 = "test_counter_1"
        name_2 = "test_counter_2"
        counters = Counters({name_1: counter_1, name_2: counter_2}, None)
        counters.expire()
        self.assertDictEqual(counters.raw, {})

    def test_expire_none(self):
        lifetime = 1000
        current_time = int(time())
        counter_1 = {
            "value": 1,
            "create_time": current_time,
            "update_time": current_time,
            "lifetime": lifetime
        }
        counter_2 = {
            "value": 2,
            "create_time": current_time,
            "update_time": current_time,
            "lifetime": lifetime
        }
        name_1 = "test_counter_1"
        name_2 = "test_counter_2"
        counters = Counters({name_1: counter_1, name_2: counter_2}, None)
        counters.expire()
        self.assertDictEqual(counters.raw, {name_1: counter_1, name_2: counter_2})

    def test_expire_not_set(self):
        current_time = int(time()) - 1
        counter_1 = {
            "value": 1,
            "create_time": current_time,
            "update_time": current_time,
            "lifetime": None
        }
        counter_2 = {
            "value": 2,
            "create_time": current_time,
            "update_time": current_time,
            "lifetime": None
        }
        name_1 = "test_counter_1"
        name_2 = "test_counter_2"
        counters = Counters({name_1: counter_1, name_2: counter_2}, None)
        counters.expire()
        self.assertDictEqual(counters.raw, {name_1: counter_1, name_2: counter_2})
